package com.company.nfp.app.calculation.product_calculations;

import com.company.nfp.app.get_entity.get_current_firm.GetCurrentFirmService;
import com.company.nfp.entity.enumerations.SourceGoodType;
import com.company.nfp.entity.enumerations.SourceRevaluationType;
import com.company.nfp.entity.not_persistent.revaluation.CostsPerDateEntity;
import com.company.nfp.entity.source.product.GoodDetail;
import com.company.nfp.entity.source.revaluation.Revaluation;
import io.jmix.core.DataManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.UUID;

@Service
public class CalcProductCostPerDateService {
    @Autowired
    private DataManager dataManager;
    @Autowired
    private GetCurrentFirmService getCurrentFirmService;

    public CostsPerDateEntity calcProductCostPerDate(UUID userId, UUID productId, LocalDateTime date) {
        GoodDetail lastPurchase;
        Revaluation lastRevaluation;
        if (userId == null) {
            try {
                lastPurchase = dataManager.load(GoodDetail.class)
                        .query("SELECT e FROM nfp_GoodDetail e WHERE e.good.sourceGoodType = :sourceGoodType AND " +
                                "e.product.id = :productId AND e.good.date < :date " +
                                "ORDER BY e.good.date DESC")
                        .parameter("sourceGoodType", SourceGoodType.ZO_RECEIPT_INVOICE.getId())
                        .parameter("productId", productId)
                        .parameter("date", date)
                        .one();
            } catch (Exception e) {
                lastPurchase = null;
            }

            try {
                lastRevaluation = dataManager.load(Revaluation.class)
                        .query("SELECT e FROM nfp_Revaluation e WHERE e.sourceRevaluationType = :sourceRevaluationType AND " +
                                "e.product.id = :productId AND e.date < :date ORDER BY e.date DESC")
                        .parameter("sourceRevaluationType", SourceRevaluationType.ZO_REVALUATION.getId())
                        .parameter("productId", productId)
                        .parameter("date", date)
                        .one();
            } catch (Exception e) {
                lastRevaluation = null;
            }
        } else {
            var currentFirm = getCurrentFirmService.getCurrentFirm(userId);
            var firmId = currentFirm.getId();
            try {
                lastPurchase = dataManager.load(GoodDetail.class)
                        .query("SELECT e FROM nfp_GoodDetail e WHERE e.good.shop.firm.id = :firmId AND " +
                                "e.good.sourceGoodType = :sourceGoodType AND " +
                                "e.product.id = :productId AND e.good.date < :date " +
                                "ORDER BY e.good.date DESC")
                        .parameter("firmId", firmId)
                        .parameter("sourceGoodType", SourceGoodType.OF_RECEIPT_INVOICE.getId())
                        .parameter("productId", productId)
                        .parameter("date", date)
                        .one();
            } catch (Exception e) {
                lastPurchase = null;
            }
            try {
                lastRevaluation = dataManager.load(Revaluation.class)
                        .query("SELECT e FROM nfp_Revaluation e WHERE e.firm.id = :firmId AND " +
                                "e.sourceRevaluationType = :sourceRevaluationType AND " +
                                "e.product.id = :productId AND e.date < :date ORDER BY e.date DESC")
                        .parameter("firmId", firmId)
                        .parameter("sourceRevaluationType", SourceRevaluationType.OF_REVALUATION.getId())
                        .parameter("productId", productId)
                        .parameter("date", date)
                        .one();
            } catch (Exception e) {
                lastRevaluation = null;
            }
        }
        return calcLastCosts(lastPurchase, lastRevaluation);
    }

    private CostsPerDateEntity calcLastCosts(GoodDetail lastPurchase, Revaluation lastRevaluation) {
        var costsPerDate = dataManager.create(CostsPerDateEntity.class);
        if (lastPurchase != null && lastRevaluation != null) {
            if (lastPurchase.getGood().getDate().isAfter(lastRevaluation.getDate())) {
                costsPerDate.setCostPrice(lastPurchase.getCostPrice());
                costsPerDate.setCost(lastPurchase.getCost());
            } else {
                costsPerDate.setCostPrice(lastRevaluation.getCostPrice());
                costsPerDate.setCost(lastRevaluation.getCost());
            }
        } else if (lastPurchase != null) {
            costsPerDate.setCostPrice(lastPurchase.getCostPrice());
            costsPerDate.setCost(lastPurchase.getCost());
        } else if (lastRevaluation != null) {
            costsPerDate.setCostPrice(lastRevaluation.getCostPrice());
            costsPerDate.setCost(lastRevaluation.getCost());

        } else {
            costsPerDate.setCostPrice(null);
            costsPerDate.setCost(null);
        }
        return costsPerDate;
    }
}