package com.company.nfp.app.calculation.product_calculations;

import io.jmix.core.DataManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.UUID;


/**
 * Service of the calculating Product Quantity Balance
 */
@Service
public class ProductQuantityBalanceService {
    @Autowired
    private DataManager dataManager;

    /**
     * Product Quantity Balance calculating for ZO
     *
     * @param dateTime A current date
     * @param productId Product Id
     * @return Product Quantity Balance
     */
    public BigDecimal productQuantityBalance(UUID productId, LocalDateTime dateTime) {
        try {
            return dataManager.loadValue("SELECT sum(e.balQuantity) FROM nfp_GoodDetail e " +
                            "WHERE e.good.date <= :dateTime AND " +
                            "e.product.id = :productId AND " +
                            "e.good.accountingType = :accountingType", BigDecimal.class)
                    .parameter("dateTime", dateTime)
                    .parameter("productId", productId)
                    .parameter("accountingType", "ZO")
                    .one();
        } catch (Exception e) {
            return BigDecimal.ZERO;
        }
    }

    public BigDecimal productQuantityBalance(UUID firmId, UUID productId, LocalDateTime dateTime) {
        try {
            return dataManager.loadValue("SELECT sum(e.balQuantity) FROM nfp_GoodDetail e " +
                            "WHERE e.good.firmAccount.firm.id = :firmId AND " +
                            "e.good.date <= :dateTime AND " +
                            "e.product.id = :productId AND " +
                            "e.good.accountingType = :accountingType", BigDecimal.class)
                    .parameter("firmId", firmId)
                    .parameter("dateTime", dateTime)
                    .parameter("productId", productId)
                    .parameter("accountingType", "OF")
                    .one();
        } catch (Exception e) {
            return BigDecimal.ZERO;
        }
    }
}