package com.company.nfp.app.copy_entity;

import com.company.nfp.app.set_next_number.SetNextNumberService;
import com.company.nfp.entity.enumerations.AccountingType;
import com.company.nfp.entity.enumerations.SourceGoodType;
import com.company.nfp.entity.source.product.Good;
import com.company.nfp.entity.source.product.GoodDetail;
import io.jmix.core.DataManager;
import io.jmix.core.MetadataTools;
import io.jmix.core.SaveContext;
import io.jmix.core.UuidProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Objects;
import java.util.UUID;

@Service
public class CopySourceEntityService {

    @Autowired
    private DataManager dataManager;
    @Autowired
    private MetadataTools metadataTools;
    @Autowired
    private SetNextNumberService setNextNumberService;

    public Good copySourceEntity(UUID userId, UUID goodId) {
        try {
            var source = dataManager.load(Good.class)
                    .id(goodId)
                    .fetchPlan("goodDcFull")
                    .one();
            SaveContext saveContext = new SaveContext();
            var copy = metadataTools.copy(source);
            copy.setId(UuidProvider.createUuid());
            copy.setDate(LocalDateTime.now());
            if (source.getGoodDetail() != null) {
                for (GoodDetail sourceDetail : source.getGoodDetail()) {
                    var copyDetail = metadataTools.copy(sourceDetail);
                    copyDetail.setId(UuidProvider.createUuid());
                    copyDetail.setGood(copy);
                    saveContext.saving(copyDetail);
                }
            }
            switch (Objects.requireNonNull(source).getSourceGoodType()) {
                case ZO_ACT:
                    setDocumentNumbersAndDates(userId, copy, SourceGoodType.OF_ACT);
                    break;
                case ZO_EXPENSE_INVOICE:
                    setDocumentNumbersAndDates(userId, copy, SourceGoodType.OF_EXPENSE_INVOICE);
                    break;
                case ZO_CASH:
                    copy.setSourceGoodType(SourceGoodType.OF_CASH);
                    copy.setAccountingType(AccountingType.OF);
                    break;
                case ZO_RECEIPT_INVOICE:
                    if (copy.getCounterparty().getIsVatPayer()) copy.setPnIsThere(true);
                    copy.setSourceGoodType(SourceGoodType.OF_RECEIPT_INVOICE);
                    copy.setAccountingType(AccountingType.OF);
                    break;
                case OF_BANK:
                    copy.setSourceGoodType(SourceGoodType.ZO_BANK);
                    copy.setAccountingType(AccountingType.ZO);
                    break;
                default: return null;
            }
            saveContext.saving(copy);
            var savedCopy = dataManager.save(saveContext);
            return savedCopy.get(copy);
        } catch (Exception e) {
            return null;
        }
    }

    private void setDocumentNumbersAndDates(UUID userId, Good copy, SourceGoodType sourceGoodType) {
        Boolean isVatPayer;
        LocalDate taxInvoiceDate;
        Long taxInvoiceNumber;
        Long documentNumber;
        copy.setSourceGoodType(sourceGoodType);
        copy.setAccountingType(AccountingType.OF);
        documentNumber = setNextNumberService.setDocumentNumber(sourceGoodType,
                userId);
        copy.setNum(documentNumber);
        copy.setNumber(String.valueOf(documentNumber));

        isVatPayer = copy.getFirmAccount().getFirm().getIsVatPayer();
        if (copy.getFirmAccount().getFirm().getIsVatPayer()) {
            taxInvoiceDate = LocalDate.now();
            taxInvoiceNumber = setNextNumberService.setTaxInvoiceNumber(taxInvoiceDate, userId);
            if (taxInvoiceNumber != null) {
                copy.setTaxInvoiceNumber(String.valueOf(taxInvoiceNumber));
                copy.setTaxInvoiceDate(taxInvoiceDate);
            }
            copy.setPnIsThere(isVatPayer);
        }
    }
}