package com.company.nfp.app.get_entity.get_document_props;

import com.company.nfp.app.get_entity.get_current_employee.GetCurrentEmployeeService;
import com.company.nfp.app.get_entity.get_current_firm.GetCurrentFirmService;
import com.company.nfp.app.get_entity.get_current_shop.GetCurrentShopService;
import com.company.nfp.app.get_entity.get_main_entity.GetMainFirmAccountService;
import com.company.nfp.entity.not_persistent.source_document.DocumentPropsEntity;
import io.jmix.core.DataManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.UUID;

@Component("nfp_GetDocumentPropsService")
public class GetDocumentPropsService {
    @Autowired
    private GetCurrentEmployeeService getCurrentEmployeeService;
    @Autowired
    private GetCurrentShopService getCurrentShopService;
    @Autowired
    private GetCurrentFirmService getCurrentFirmService;
    @Autowired
    private GetMainFirmAccountService getMainFirmAccountService;
    @Autowired
    private DataManager dataManager;

    /**
     * Get Document Current Data:
     *      1. Current Employee
     *      2. Current Shop
     *      3. CurrentFirm
     *      4. Main Firm Account
     * @param userId The Current User Id
     * @return DocumentPropsEntity instance
     */
    public DocumentPropsEntity getDocumentProps (UUID userId) {
        var employee = getCurrentEmployeeService.getCurrentEmployee(userId);
        var shop = getCurrentShopService.getCurrentShop(userId);
        var firm = getCurrentFirmService.getCurrentFirm(userId);
        var account = getMainFirmAccountService.getMain(firm.getId());
        if (account == null || shop == null || employee == null) {
            return null;
        } else {
            var docProps = dataManager.create(DocumentPropsEntity.class);
            docProps.setEmployee(employee);
            docProps.setShop(shop);
            docProps.setAccount(account);
            return docProps;
        }
    }
}