package com.company.nfp.app.get_entity.get_last_purchase;

import com.company.nfp.app.get_entity.get_current_firm.GetCurrentFirmService;
import com.company.nfp.entity.enumerations.SourceGoodType;
import com.company.nfp.entity.source.product.GoodDetail;
import io.jmix.core.DataManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@Service
public class GetLastCostsOfService {
    @Autowired
    private DataManager dataManager;
    @Autowired
    private GetCurrentFirmService getCurrentFirmService;
    @Autowired
    private SetCostsForIsServiceService setCostsForIsServiceService;

    public Map<String, BigDecimal> getLastCostsOf(UUID userId, UUID productId, LocalDateTime dateTime) {
        Map<String, BigDecimal> map = new HashMap<>();
        var mapIsService = setCostsForIsServiceService.setCostsForIsService(productId);
        if (mapIsService != null) return mapIsService;
        GoodDetail detail;
        var currentFirm = getCurrentFirmService.getCurrentFirm(userId);
        var firmId = currentFirm.getId();
        try {
            detail = dataManager.load(GoodDetail.class)
                    .query("SELECT e FROM nfp_GoodDetail e WHERE " +
                            "e.product.id = :productId AND " +
                            "e.good.date <= :dateTime AND " +
                            "e.good.sourceGoodType = :sourceGoodType AND " +
                            "e.good.firmAccount.firm.id = :firmId " +
                            "ORDER BY e.good.date DESC")
                    .parameter("productId", productId)
                    .parameter("dateTime", dateTime)
                    .parameter("sourceGoodType", SourceGoodType.OF_RECEIPT_INVOICE)
                    .parameter("firmId", firmId)
                    .one();
            map.put("costPrice", detail.getCostPrice());
            map.put("cost", detail.getCost());
            return map;
        } catch (Exception e) {
            return null;
        }
    }
}