package com.company.nfp.app.get_entity.get_last_purchase;

import com.company.nfp.entity.enumerations.SourceGoodType;
import com.company.nfp.entity.source.product.GoodDetail;
import io.jmix.core.DataManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@Service
public class GetLastCostsZoService {
    @Autowired
    private DataManager dataManager;
    @Autowired
    private SetCostsForIsServiceService setCostsForIsServiceService;

    /**
     * Get costs from last zo purchase
     *
     * @param productId Product Id of the product looking for
     * @param dateTime  Date of the current document
     * @return HashMap with keys "costPrice", "cost"
     */
    public Map<String, BigDecimal> getLastCostsZo(UUID productId, LocalDateTime dateTime) {
        Map<String, BigDecimal> map = new HashMap<>();
        var mapIsService = setCostsForIsServiceService.setCostsForIsService(productId);
        if (mapIsService != null) return mapIsService;
        GoodDetail detail;
        try {
            detail = dataManager.load(GoodDetail.class)
                    .query("SELECT e FROM nfp_GoodDetail e WHERE " +
                            "e.product.id = :productId AND " +
                            "e.good.date <= :dateTime AND " +
                            "e.good.sourceGoodType = :sourceGoodType " +
                            "ORDER BY e.good.date DESC")
                    .parameter("productId", productId)
                    .parameter("dateTime", dateTime)
                    .parameter("sourceGoodType", SourceGoodType.ZO_RECEIPT_INVOICE)
                    .one();
            map.put("costPrice", detail.getCostPrice());
            map.put("cost", detail.getCost());
            return map;
        } catch (Exception e) {
            return null;
        }
    }

/*
    "SELECT e FROM mtfp_PurchaseInvoiceDetail e WHERE " +
            "e.product.id = :productId AND " +
            "e.purchaseInvoice.date < :date " +
            "ORDER BY e.purchaseInvoice.date DESC"
*/
}