package com.company.nfp.app.import_data.from_excel;

import com.company.nfp.app.check_db.CheckFullDbService;
import com.company.nfp.app.import_data.koatuu.CreateKoatuuService;
import com.company.nfp.app.import_data.koatuu.SaveKoatuuEntitiesService;
import com.company.nfp.entity.dictionaries.koatuu.Koatuu;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Objects;
import org.apache.poi.ss.usermodel.Row;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * Import KOATUU Service
 */
@Service
public class ImportFromExcelService {

  @Autowired
  private OpenBookService openBookService;
  @Autowired
  private GetSheetService getSheetService;
  @Autowired
  private GetRowsService getRowsService;
  @Autowired
  private GetCellsService getCellsService;
  @Autowired
  private CreateKoatuuService createKoatuuService;
  @Autowired
  private SaveKoatuuEntitiesService saveKoatuuEntitiesService;

  //region Local Variables
  private String result;
  @Autowired
  private CheckFullDbService checkFullDbService;
  //endregion


  public String factory(String filePath, String sheetName, String entityName) throws IOException {

    if (!Objects.equals(checkFullDbService.checkFullDb(entityName), "Ok")) return "com.company.nfp.checkDb/full";

    switch (entityName) {
      case "Koatuu":
        try {
          if (filePath == null || sheetName == null) {
            return "notifications.warning.invalidOperation";
          }
          var book = openBookService.openBook(filePath);
          if (book == null) {
            return "notifications.warning.invalidOperation";
          }

          var sheet = getSheetService.getSheet(book, sheetName);
          if (sheet == null) {
            return "notifications.warning.invalidOperation";
          }

          var rows = getRowsService.getRows(sheet);
          if (rows == null) {
            return "notifications.warning.invalidOperation";
          }

          var listKoatuu = new ArrayList<Koatuu>();
          for (Row row : rows) {
            ArrayList<Object> cells = getCellsService.getCells(row);
            var koatuu = createKoatuuService.createKoatuu(cells);
            listKoatuu.add(koatuu);
          }
          result = saveKoatuuEntitiesService.saveKoatuuEntities(listKoatuu);

        } catch (Exception e) {
          return "notifications.warning.invalidOperation";
        }
        break;
    }
    return result;
  }
}