package com.company.nfp.app.prro.check_prro_status;

import com.company.nfp.app.prro.get_prro_state.GetPrroStateService;
import com.company.nfp.entity.not_persistent.prro.ResultCheckPrroStatusEntity;
import io.jmix.core.DataManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.UUID;

import static java.lang.Math.abs;

@Service
public class CheckPrroStatusService {
    @Autowired
    private DataManager dataManager;
    @Autowired
    private GetPrroStateService getPrroStateService;

    public ResultCheckPrroStatusEntity checkPrroStatus(UUID userId) throws IOException, InterruptedException {
        var result = dataManager.create(ResultCheckPrroStatusEntity.class);
        var prroState = getPrroStateService.getState(userId);
        if (prroState == null || prroState.getStatus() != 200) {
            result.setMessage("msg://com.company.nfp.notification/noServerConnection");
        } else {
            if (!prroState.getIsShiftOpen()) {
                result.setMessage("com.company.nfp.notification/isShiftClose");
                result.setShiftOpen(false);
                return result;
            }
            var formatter = DateTimeFormatter.ofPattern("dd.MM.uuuu HH:mm:ss");
            var shiftOpenDate = LocalDateTime.parse(prroState.getShiftOpenDate(), formatter);
            var nowLocalDateTime = LocalDateTime.now();
            var duration = Duration.between(nowLocalDateTime, shiftOpenDate);
            var durationToMinutes = Float.valueOf(duration.toMinutes());
            var coefficient = (abs(durationToMinutes) / 60F);
            if (coefficient > 23 && coefficient < 24) {
                result.setDurationToMinutes(24 * 60 - abs(duration.toMinutes()));
                result.setMessage("com.company.nfp.notification/needCloseShift");
                result.setShiftOpen(true);
            } else if (coefficient >= 24) {
                result.setMessage("com.company.nfp.notification/needCloseShift");
            } else if (coefficient < 23) {
                result.setShiftOpen(true);
            }
        }
        return result;
    }
}