package com.company.nfp.app.prro.in_out_cash;

import com.company.nfp.app.get_entity.get_current_employee.GetCurrentEmployeeService;
import com.company.nfp.app.prro.helpers.UrlStringCreateService;
import com.company.nfp.entity.enumerations.DocSubType;
import com.company.nfp.entity.not_persistent.prro.ServiceInOutEntity;
import com.google.gson.Gson;
import io.jmix.core.DataManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.math.BigDecimal;
import java.util.UUID;

@Service
public class InOutCashService {
    @Autowired
    private DataManager dataManager;
    @Autowired
    private GetCurrentEmployeeService getCurrentEmployeeService;
    @Autowired
    private UrlStringCreateService urlStringCreateService;

    public Integer executeRequest(DocSubType docSubType, BigDecimal sum, UUID userId) {
        try {
            Integer status;
            ServiceInOutEntity serviceInOutEntity = createDto(docSubType, sum, userId);
            var gson = new Gson();
            var body = gson.toJson(serviceInOutEntity);
            String path = null;
            path = getPath(docSubType, path);
            var urlString = urlStringCreateService.urlStringCreate(userId, path);
            var rest = new RestTemplate();
            var headers = new HttpHeaders();
            headers.add("Content-Type", "application/json;charset=utf8");
            HttpEntity<String> requestEntity = new HttpEntity<>(body, headers);

            var responseEntity = rest.exchange(
                    urlString,
                    HttpMethod.POST,
                    requestEntity,
                    String.class
                );
            HttpStatus httpStatus = responseEntity.getStatusCode();
            status = httpStatus.value();
            //var response = responseEntity.getBody();
            return status;
        } catch (RestClientException e) {
            return 1000;
        }
    }

    private ServiceInOutEntity createDto(DocSubType docSubType, BigDecimal sum, UUID userId) {
        var serviceInOutEntity = dataManager.create(ServiceInOutEntity.class);
        serviceInOutEntity.setDocSubType(docSubType.getId());
        serviceInOutEntity.setSum(sum.longValue());
        var employee = getCurrentEmployeeService.getCurrentEmployee(userId);
        if (employee != null) {
            serviceInOutEntity.setCashierName(employee.getFullName());
        }
        return serviceInOutEntity;
    }

    private String getPath(DocSubType docSubType, String path) {
        switch (docSubType) {
            case CASH_DEPOSIT:
                path = "innovate/servicein";
                break;
            case CASH_DROP:
                path = "innovate/serviceout";
                break;
        }
        return path;
    }
}