package com.company.nfp.app.prro.receipt;

import com.company.nfp.app.get_entity.get_check_by_id.GetCheckByIdService;
import com.company.nfp.app.prro.commit_entity.CheckCommitService;
import com.company.nfp.app.prro.helpers.GetCheckDfsLinkService;
import com.company.nfp.app.prro.helpers.GetResponseService;
import com.company.nfp.app.prro.helpers.UrlStringCreateService;
import com.company.nfp.entity.enumerations.DocSubType;
import com.company.nfp.entity.not_persistent.prro.SaleReturnReceiptEntity;
import com.company.nfp.entity.source.product.Good;
import com.company.nfp.entity.source.product.GoodDetail;
import com.google.gson.Gson;
import io.jmix.core.DataManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@Service
public class SaleReceiptService {
    @Autowired
    private GetCheckByIdService getCheckByIdService;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private UrlStringCreateService urlStringCreateService;
    @Autowired
    private GetResponseService getResponseService;
    @Autowired
    private GetCheckDfsLinkService getCheckDfsLinkService;
    @Autowired
    private CheckCommitService checkCommitService;

    public SaleReturnReceiptEntity executeRequest(DocSubType docSubType, UUID checkId) throws IOException, InterruptedException {
        var check = getCheckByIdService.getCheck(checkId);   // get current check
        if (check == null) {
            var saleReturnReceiptEntity = dataManager.create(SaleReturnReceiptEntity.class);
            saleReturnReceiptEntity.setStatus(0);
            return saleReturnReceiptEntity;
        }
        var number = 0;
        var saleRequest = saleRequestCreate(check, docSubType);
        bodyTypeOneCreate(check, number, saleRequest);

        var gson = new Gson();
        var body = gson.toJson(saleRequest);
        var userId = check.getEmployee().getUser().getId();
        String urlString;
        SaleReturnReceiptEntity response = null;
        switch (docSubType) {
            case TEXT:
                urlString = urlStringCreateService.urlStringCreate(userId, "innovate/printreceipt");
                response = getResponseService.getResponse(body, urlString);
                check.setDocSubType(DocSubType.TEXT);
                check.setFn(response.getFiscalNumber());
                check.setQr(response.getQR());
                check.setDfsLink(getCheckDfsLinkService.getLink(check.getQr()));
                check.setCheckText(response.getText());
                checkCommitService.checkCommit(check);
                return response;
            case NON_FISCAL_RECEIPT:
                urlString = urlStringCreateService.urlStringCreate(userId, "innovate/printnonfiscalreceipt");
                response = getResponseService.getResponse(body, urlString);
                check.setDocSubType(DocSubType.NON_FISCAL_RECEIPT);
                check.setCheckText(response.getText());
                checkCommitService.checkCommit(check);
                return response;
        }
        return response;
    }
    public SaleReturnReceiptEntity executeRequest(DocSubType docSubType, UUID checkId, String fn) throws IOException, InterruptedException {
        var check = getCheckByIdService.getCheck(checkId);   // get current check
        if (check == null) {
            var saleReturnReceiptEntity = dataManager.create(SaleReturnReceiptEntity.class);
            saleReturnReceiptEntity.setStatus(0);
            return saleReturnReceiptEntity;
        }
        var number = 0;
        var returnRequest = returnRequestCreate(check, docSubType, fn);
        number = bodyTypeOneCreate(check, number, returnRequest);
        bodyTypeTwoCreate(check, number, returnRequest);
        var gson = new Gson();
        var body = gson.toJson(returnRequest);
        var userId = check.getEmployee().getUser().getId();
        var urlString = urlStringCreateService.urlStringCreate(userId, "innovate/printreceipt");
        var response = getResponseService.getResponse(body, urlString);
        check.setDocSubType(DocSubType.RECEIPT_POSITION);
        check.setFn(response.getFiscalNumber());
        check.setQr(response.getQR());
        check.setDfsLink(getCheckDfsLinkService.getLink(check.getQr()));
        check.setCheckText(response.getText());
        checkCommitService.checkCommit(check);
        return response;
    }

    private void bodyTypeTwoCreate(Good check, int number, ReturnRequest returnRequest) {
        var number1 = number;
        var bodyTypeTwo = new BodyTypeTwo();
        bodyTypeTwo.type = 2;
        bodyTypeTwo.sum = check.getSum().floatValue();
        bodyTypeTwo.returnSum = 0F;
        bodyTypeTwo.name = "ГОТІВКА";
        bodyTypeTwo.typePay = 0;
        number1 += 1;
        bodyTypeTwo.number = number1;
        returnRequest.body.add(bodyTypeTwo);
    }

    private int bodyTypeOneCreate(Good check, int number, ReturnRequest returnRequest) {
        var number1 = number;
        if (check.getGoodDetail() != null) {
            for (GoodDetail checkDetail : check.getGoodDetail()) {
                var returnBodyTypeOne = new ReturnBodyTypeOne();
                returnBodyTypeOne.type = 1;
                returnBodyTypeOne.sku = checkDetail.getProduct().getCode().intValue();
                returnBodyTypeOne.name = checkDetail.getProduct().getFullName();
                returnBodyTypeOne.price = checkDetail.getCost().floatValue();
                returnBodyTypeOne.quantity = checkDetail.getQuantity().floatValue();
                returnBodyTypeOne.discountSum = 0F;
                returnBodyTypeOne.amount = checkDetail.getSum().floatValue();
                returnBodyTypeOne.vatGroup = checkDetail.getProduct().getVatGroup();
                returnBodyTypeOne.codeUnit = Integer.valueOf(checkDetail.getProduct().getUnit().getNum());
                returnBodyTypeOne.unitName = checkDetail.getProduct().getUnit().getShortName();
                number1 += 1;
                returnBodyTypeOne.number = number1;
                returnRequest.body.add(returnBodyTypeOne);
            }
        }
        return number1;
    }

    private ReturnRequest returnRequestCreate(Good check, DocSubType docSubType, String fn) {
        var returnRequest = new ReturnRequest();
        returnRequest.id = check.getId().toString();
        returnRequest.sourceFiscalNumber = fn;
        returnRequest.docSubType = docSubType.getId();
        returnRequest.roundDiscount = 0F;
        returnRequest.cashierName = check.getEmployee().getFullName();
        returnRequest.sum = check.getSum().floatValue();
        return returnRequest;
    }

    private void bodyTypeOneCreate(Good check, int number, SaleRequest saleRequest) {
        var number1 = number;
        if (check.getByCash().compareTo(BigDecimal.ZERO) != 0) {
            var bodyTypeTwo = new BodyTypeTwo();
            bodyTypeTwo.type = 2;
            bodyTypeTwo.sum = check.getByCash().subtract(check.getChange()).floatValue();
            bodyTypeTwo.returnSum = check.getChange().floatValue();
            bodyTypeTwo.name = "ГОТІВКА";
            bodyTypeTwo.typePay = 0;
            number1 += 1;
            bodyTypeTwo.number = number1;
            saleRequest.body.add(bodyTypeTwo);
        }
        if (check.getByCard().compareTo(BigDecimal.ZERO) != 0) {
            var bodyTypeTwo = new BodyTypeTwo();
            bodyTypeTwo.type = 2;
            bodyTypeTwo.sum = check.getByCard().floatValue();
            bodyTypeTwo.returnSum = 0F;
            bodyTypeTwo.name = "КАРТКА";
            bodyTypeTwo.typePay = 3;
            number1 += 1;
            bodyTypeTwo.number = number1;
            saleRequest.body.add(bodyTypeTwo);
        }

        if (check.getByCredit().compareTo(BigDecimal.ZERO) != 0) {
            var bodyTypeTwo = new BodyTypeTwo();
            bodyTypeTwo.type = 2;
            bodyTypeTwo.sum = check.getByCredit().floatValue();
            bodyTypeTwo.returnSum = 0F;
            bodyTypeTwo.name = "КРЕДИТ";
            bodyTypeTwo.typePay = 1;
            number1 += 1;
            bodyTypeTwo.number = number1;
            saleRequest.body.add(bodyTypeTwo);
        }

        if (check.getByCheck().compareTo(BigDecimal.ZERO) != 0) {
            var bodyTypeTwo = new BodyTypeTwo();
            bodyTypeTwo.type = 2;
            bodyTypeTwo.sum = check.getByCheck().floatValue();
            bodyTypeTwo.returnSum = 0F;
            bodyTypeTwo.name = "ЧЕК";
            bodyTypeTwo.typePay = 2;
            number1 += 1;
            bodyTypeTwo.number = number1;
            saleRequest.body.add(bodyTypeTwo);
        }

        if (check.getByBonuses().compareTo(BigDecimal.ZERO) != 0) {
            var bodyTypeTwo = new BodyTypeTwo();
            bodyTypeTwo.type = 2;
            bodyTypeTwo.sum = check.getByBonuses().floatValue();
            bodyTypeTwo.returnSum = 0F;
            bodyTypeTwo.name = "БОНУСИ";
            bodyTypeTwo.typePay = 4;
            number1 += 1;
            bodyTypeTwo.number = number1;
            saleRequest.body.add(bodyTypeTwo);
        }

    }

    /**
     * Create a receipt header
     *
     * @param check      The check
     * @param docSubType The Check Type
     * @return SaleRequest
     * @see SaleRequest
     */
    private SaleRequest saleRequestCreate(Good check, DocSubType docSubType) {
        var saleRequest = new SaleRequest();
        saleRequest.id = check.getId().toString();
        saleRequest.docSubType = docSubType.getId();
        saleRequest.roundDiscount = 0F;
        saleRequest.cashierName = check.getEmployee().getFullName();
        saleRequest.sum = check.getSum().floatValue();
        return saleRequest;
    }

    private class SaleRequest {
        String id = null;
        String docSubType = null;
        Float sum = null;
        Float roundDiscount = null;
        String cashierName = null;
        List<Object> body = new ArrayList<>();
    }

    private static class ReturnRequest {
        String id = null;
        String docSubType = null;
        Float sum = null;
        Float roundDiscount = null;
        String sourceFiscalNumber = null;
        String cashierName = null;
        List<Object> body = new ArrayList<>();
    }

    private static class BodyTypeZero {
        Integer type = null;
        String text = null;
        Integer number = null;
    }

    private static class SaleBodyTypeOne {
        Integer type = null;
        Integer sku = null;
        String name = null;
        Float price = null;
        Float quantity = null;
        Float discountSum = null;
        String UKTZED = null;
        Float amount = null;
        String vatGroup = null;
        String registeredArticle = null;
        Integer codeUnit = null;
        String unitName = null;
        Integer number = null;
        List<Object> excises = new ArrayList<>();
    }

    private static class ReturnBodyTypeOne {
        Integer type = null;
        Integer sku = null;
        String name = null;
        Float price = null;
        Float quantity = null;
        Float discountSum = null;
        Float amount = null;
        String vatGroup = null;
        Integer codeUnit = null;
        String unitName = null;
        Integer number = null;
    }

    private class BodyTypeTwo {
        Integer type = null;
        Float sum = null;
        Float returnSum = null;
        String name = null;
        Integer typePay = null;
        Integer number = null;
    }
}