package com.company.nfp.app.prro.x_report;

import com.company.nfp.app.get_entity.get_current_employee.GetCurrentEmployeeService;
import com.company.nfp.app.prro.helpers.UrlStringCreateService;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.UUID;

@Service
public class XReportService {
    @Autowired
    private GetCurrentEmployeeService getCurrentEmployeeService;
    @Autowired
    private UrlStringCreateService urlStringCreateService;

    public String executeRequest(UUID userId, String path) throws URISyntaxException, IOException, InterruptedException {
        String result;
        try {
            var requestAttribute = new CashierName();
            var employee = getCurrentEmployeeService.getCurrentEmployee(userId);
            if (employee != null) requestAttribute.setCashierName(employee.getFullName());
            var gson = new Gson();
            var body = gson.toJson(requestAttribute);
            var urlString = urlStringCreateService.urlStringCreate(userId, path);

            var client = HttpClient.newBuilder().build();
            var request = HttpRequest.newBuilder().POST(HttpRequest.BodyPublishers.ofString(body))
                    .uri(new URI(urlString))
                    .header("Content-Type", "application/json;charset=utf-8")
                    .build();
            var response = client.send(request, HttpResponse.BodyHandlers.ofString());
            var reportText = gson.fromJson(response.body(), ReportText.class);
            result = reportText.text;
        } catch (URISyntaxException | IOException | InterruptedException | JsonSyntaxException e) {
            result = "Error";
        }
        return result;
    }

    private class CashierName {
        private String cashierName = null;

        public void setCashierName(String cashierName) {
            this.cashierName = cashierName;
        }
    }

    private class ReportText {
        private String text = null;
    }
}