package com.company.nfp.entity.dictionaries.firm;

import com.company.nfp.entity.RegEusPayer;
import com.company.nfp.entity.dictionaries.dpi.Dpi;
import com.company.nfp.entity.dictionaries.firm.firm_account.FirmAccount;
import com.company.nfp.entity.enumerations.TaxationSystem;
import com.company.nfp.entity.mapped_super_class.TenantNamed;
import io.jmix.core.DeletePolicy;
import io.jmix.core.entity.annotation.OnDelete;
import io.jmix.core.metamodel.annotation.Composition;
import io.jmix.core.metamodel.annotation.DependsOnProperties;
import io.jmix.core.metamodel.annotation.InstanceName;
import io.jmix.core.metamodel.annotation.JmixEntity;

import javax.persistence.*;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;

@JmixEntity
@Table(name = "NFP_FIRM")
@Entity(name = "nfp_Firm")
public class Firm extends TenantNamed {

  @Column(name = "IPN")
  private String ipn;

  @Column(name = "CURRENT_LIVING_WAGE")
  private BigDecimal currentLivingWage;

  @Column(name = "CURRENT_MINIMUM_SALARY")
  private BigDecimal currentMinimumSalary;

  @Column(name = "TAX_NUMBER", nullable = false)
  private String taxNumber;

  @Column(name = "TAXPAYER_REG_DATE")
  private LocalDate taxpayerRegDate;

  @Column(name = "TAXPAYER_REG_NUMBER")
  private String taxpayerRegNumber;

  @Column(name = "SPECIAL_MODE")
  private String specialMode;

  @Column(name = "DEREG_DATE")
  private LocalDate deregDate;

  @Column(name = "IS_VAT_PAYER")
  private Boolean isVatPayer;

  @Column(name = "ADDRESS")
  @Lob
  private String address;

  @Column(name = "PHONES")
  private String phones;

  @OnDelete(DeletePolicy.CASCADE)
  @Composition
  @OneToMany(mappedBy = "firm")
  private List<FirmKved> firmKved;

  @OnDelete(DeletePolicy.CASCADE)
  @Composition
  @OneToMany(mappedBy = "firm")
  private List<Shop> shops;

  @Composition
  @OnDelete(DeletePolicy.CASCADE)
  @OneToMany(mappedBy = "firm")
  private List<RegSingleTaxPayer> regSingleTaxPayers;

  @Composition
  @OnDelete(DeletePolicy.CASCADE)
  @OneToMany(mappedBy = "firm")
  private List<RegEusPayer> regEusPayers;

  @OnDelete(DeletePolicy.CASCADE)
  @Composition
  @OneToMany(mappedBy = "firm")
  private List<Rro> rro;

  @OnDelete(DeletePolicy.CASCADE)
  @Composition
  @OneToMany(mappedBy = "firm")
  private List<RegVatPayer> regVatPayers;

  @Column(name = "HEAD_POSITION")
  private String headPosition;

  @Column(name = "HEAD_NAME")
  private String headName;

  @Column(name = "HEAD_NAME_IN_TEXT")
  private String headNameInText;

  @Column(name = "HEAD_NAME_IN_SIGN")
  private String headNameInSign;

  @Column(name = "HEAD_SHORT_NAME")
  private String headShortName;

  @Column(name = "ACCOUNTANT_NAME")
  private String accountantName;

  @Column(name = "HEAD_PHONE")
  private String headPhone;

  @Column(name = "ACCOUNTANT_PHONE")
  private String accountantPhone;

  @Column(name = "FIRM_EMAIL")
  private String firmEmail;

  @Column(name = "TAX_RATE_FO")
  private BigDecimal taxRateFo;

  @Column(name = "TAXATION_SYSTEM")
  private String taxationSystem;

  @Column(name = "ESV")
  private BigDecimal esv;

  @Column(name = "GENERAL_SYSTEM_ESV")
  private BigDecimal generalSystemEsv;

  @Column(name = "GENERAL_SYSTEM_INCOME_TAX")
  private BigDecimal generalSystemIncomeTax;

  @Column(name = "GENERAL_SYSTEM_MILITARY_COLLECTION")
  private BigDecimal generalSystemMilitaryCollection;

  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "dpi_id")
  private Dpi dpi;

  @OnDelete(DeletePolicy.CASCADE)
  @Composition
  @OneToMany(mappedBy = "firm")
  private List<FirmAccount> firmAccount;

  public Boolean getIsVatPayer() {
    return isVatPayer;
  }

  public void setIsVatPayer(Boolean isVatPayer) {
    this.isVatPayer = isVatPayer;
  }

  public List<FirmAccount> getFirmAccount() {
    return firmAccount;
  }

  public void setFirmAccount(List<FirmAccount> firmAccount) {
    this.firmAccount = firmAccount;
  }

  public List<Shop> getShops() {
    return shops;
  }

  public void setShops(List<Shop> shops) {
    this.shops = shops;
  }

  public void setFirmEmail(String firmEmail) {
    this.firmEmail = firmEmail;
  }

  public String getFirmEmail() {
    return firmEmail;
  }

  public void setAccountantPhone(String accountantPhone) {
    this.accountantPhone = accountantPhone;
  }

  public String getAccountantPhone() {
    return accountantPhone;
  }

  public void setHeadPhone(String headPhone) {
    this.headPhone = headPhone;
  }

  public String getHeadPhone() {
    return headPhone;
  }

  public Dpi getDpi() {
    return dpi;
  }

  public void setDpi(Dpi dpi) {
    this.dpi = dpi;
  }

  public void setIpn(String ipn) {
    this.ipn = ipn;
  }

  public String getIpn() {
    return ipn;
  }

  public void setGeneralSystemMilitaryCollection(BigDecimal generalSystemMilitaryCollection) {
    this.generalSystemMilitaryCollection = generalSystemMilitaryCollection;
  }

  public BigDecimal getGeneralSystemMilitaryCollection() {
    return generalSystemMilitaryCollection;
  }

  public void setGeneralSystemIncomeTax(BigDecimal generalSystemIncomeTax) {
    this.generalSystemIncomeTax = generalSystemIncomeTax;
  }

  public BigDecimal getGeneralSystemIncomeTax() {
    return generalSystemIncomeTax;
  }

  public void setGeneralSystemEsv(BigDecimal generalSystemEsv) {
    this.generalSystemEsv = generalSystemEsv;
  }

  public BigDecimal getGeneralSystemEsv() {
    return generalSystemEsv;
  }

  public void setEsv(BigDecimal esv) {
    this.esv = esv;
  }

  public BigDecimal getEsv() {
    return esv;
  }

  public void setTaxRateFo(BigDecimal taxRateFo) {
    this.taxRateFo = taxRateFo;
  }

  public BigDecimal getTaxRateFo() {
    return taxRateFo;
  }

  public TaxationSystem getTaxationSystem() {
    return taxationSystem == null ? null : TaxationSystem.fromId(taxationSystem);
  }

  public void setTaxationSystem(TaxationSystem taxationSystem) {
    this.taxationSystem = taxationSystem == null ? null : taxationSystem.getId();
  }

  public void setCurrentMinimumSalary(BigDecimal currentMinimumSalary) {
    this.currentMinimumSalary = currentMinimumSalary;
  }

  public BigDecimal getCurrentMinimumSalary() {
    return currentMinimumSalary;
  }

  public void setCurrentLivingWage(BigDecimal currentLivingWage) {
    this.currentLivingWage = currentLivingWage;
  }

  public BigDecimal getCurrentLivingWage() {
    return currentLivingWage;
  }

  public void setRegEusPayers(List<RegEusPayer> regEusPayers) {
    this.regEusPayers = regEusPayers;
  }

  public List<RegEusPayer> getRegEusPayers() {
    return regEusPayers;
  }

  public void setRegSingleTaxPayers(
      List<RegSingleTaxPayer> regSingleTaxPayers) {
    this.regSingleTaxPayers = regSingleTaxPayers;
  }

  public List<RegSingleTaxPayer> getRegSingleTaxPayers() {
    return regSingleTaxPayers;
  }

  public List<RegVatPayer> getRegVatPayers() {
    return regVatPayers;
  }

  public void setRegVatPayers(
      List<RegVatPayer> regVatPayers) {
    this.regVatPayers = regVatPayers;
  }

  public List<Rro> getRro() {
    return rro;
  }

  public void setRro(List<Rro> rro) {
    this.rro = rro;
  }


  public String getAddress() {
    return address;
  }

  public List<FirmKved> getFirmKved() {
    return firmKved;
  }

  public void setFirmKved(List<FirmKved> firmKved) {
    this.firmKved = firmKved;
  }

  public String getPhones() {
    return phones;
  }

  public void setPhones(String phones) {
    this.phones = phones;
  }

  public void setAddress(String address) {
    this.address = address;
  }

  public LocalDate getDeregDate() {
    return deregDate;
  }

  public void setDeregDate(LocalDate deregDate) {
    this.deregDate = deregDate;
  }

  public String getSpecialMode() {
    return specialMode;
  }

  public void setSpecialMode(String specialMode) {
    this.specialMode = specialMode;
  }

  public String getTaxpayerRegNumber() {
    return taxpayerRegNumber;
  }

  public void setTaxpayerRegNumber(String taxpayerRegNumber) {
    this.taxpayerRegNumber = taxpayerRegNumber;
  }

  public LocalDate getTaxpayerRegDate() {
    return taxpayerRegDate;
  }

  public void setTaxpayerRegDate(LocalDate taxpayerRegDate) {
    this.taxpayerRegDate = taxpayerRegDate;
  }


  public String getTaxNumber() {
    return taxNumber;
  }

  public void setTaxNumber(String taxNumber) {
    this.taxNumber = taxNumber;
  }

  public String getHeadPosition() {
    return headPosition;
  }

  public void setHeadPosition(String headPosition) {
    this.headPosition = headPosition;
  }

  public String getHeadName() {
    return headName;
  }

  public void setHeadName(String headName) {
    this.headName = headName;
  }

  public String getHeadNameInText() {
    return headNameInText;
  }

  public void setHeadNameInText(String headNameInText) {
    this.headNameInText = headNameInText;
  }

  public String getHeadNameInSign() {
    return headNameInSign;
  }

  public void setHeadNameInSign(String headNameInSign) {
    this.headNameInSign = headNameInSign;
  }

  public String getHeadShortName() {
    return headShortName;
  }

  public void setHeadShortName(String headShortName) {
    this.headShortName = headShortName;
  }

  public String getAccountantName() {
    return accountantName;
  }

  public void setAccountantName(String accountantName) {
    this.accountantName = accountantName;
  }

  @InstanceName
  @DependsOnProperties({"fullName"})
  public String getInstanceName() {
    return String.format("%s", getFullName());
  }
}