package com.company.nfp.entity.dictionaries.product;

import com.company.nfp.entity.mapped_super_class.TenantNamed;
import io.jmix.core.metamodel.annotation.DependsOnProperties;
import io.jmix.core.metamodel.annotation.InstanceName;
import io.jmix.core.metamodel.annotation.JmixEntity;
import io.jmix.core.metamodel.annotation.JmixProperty;

import javax.persistence.*;
import java.math.BigDecimal;

@JmixEntity
@Table(name = "NFP_PRODUCT", indexes = {
        @Index(name = "IDX_NFPPRODUCT_PRODUCTSUBGROUP", columnList = "PRODUCT_SUBGROUP_ID"),
        @Index(name = "IDX_NFP_PRODUCT_UKTZED", columnList = "UKTZED_ID"),
        @Index(name = "IDX_NFP_PRODUCT_UNIT", columnList = "UNIT_ID")
})
@Entity(name = "nfp_Product")
public class Product extends TenantNamed {


    @Column(name = "BARCODE")
    private String barcode;

    @Column(name = "CODE")
    private Long code;

    @Column(name = "VAT_GROUP")
    private String vatGroup = "A";

    @Column(name = "IS_EXCISE")
    private Boolean isExcise = false;

    @Column(name = "IS_SERVICE")
    private Boolean isService = false;

    @JoinColumn(name = "PRODUCT_SUBGROUP_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private ProductSubgroup productSubgroup;

    @JoinColumn(name = "UKTZED_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private Uktzed uktzed;

    @JoinColumn(name = "UNIT_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private Unit unit;

    @JmixProperty
    @Transient
    private BigDecimal balance;

    @JmixProperty
    @Transient
    private BigDecimal costPrice;

    @JmixProperty
    @Transient
    private BigDecimal costPurchase;

    public Unit getUnit() {
        return unit;
    }

    public void setUnit(Unit unit) {
        this.unit = unit;
    }

    public Uktzed getUktzed() {
        return uktzed;
    }

    public void setUktzed(Uktzed uktzed) {
        this.uktzed = uktzed;
    }

    public ProductSubgroup getProductSubgroup() {
        return productSubgroup;
    }

    public void setProductSubgroup(ProductSubgroup productSubgroup) {
        this.productSubgroup = productSubgroup;
    }

    public BigDecimal getCostPurchase() {
        return costPurchase;
    }

    public void setCostPurchase(BigDecimal costPurchase) {
        this.costPurchase = costPurchase;
    }

    public BigDecimal getCostPrice() {
        return costPrice;
    }

    public void setCostPrice(BigDecimal costPrice) {
        this.costPrice = costPrice;
    }

    public BigDecimal getBalance() {
        return balance;
    }

    public void setBalance(BigDecimal balance) {
        this.balance = balance;
    }

    public Boolean getIsService() {
        return isService;
    }

    public void setIsService(Boolean isService) {
        this.isService = isService;
    }

    public Boolean getIsExcise() {
        return isExcise;
    }

    public void setIsExcise(Boolean isExcise) {
        this.isExcise = isExcise;
    }

    public String getVatGroup() {
        return vatGroup;
    }

    public void setVatGroup(String vatGroup) {
        this.vatGroup = vatGroup;
    }

    public Long getCode() {
        return code;
    }

    public void setCode(Long code) {
        this.code = code;
    }

    public String getBarcode() {
        return barcode;
    }

    public void setBarcode(String barcode) {
        this.barcode = barcode;
    }

    @InstanceName
    @DependsOnProperties({"code", "fullName"})
    public String getInstanceName() {
        return String.format("%s :: %s",getCode(), getFullName());
    }
}