package com.company.nfp.entity.source.product;

import com.company.nfp.entity.dictionaries.counterparty.Counterparty;
import com.company.nfp.entity.dictionaries.employee.Employee;
import com.company.nfp.entity.dictionaries.firm.Shop;
import com.company.nfp.entity.dictionaries.firm.firm_account.FirmAccount;
import com.company.nfp.entity.dictionaries.service.Service;
import com.company.nfp.entity.dictionaries.stat.Stat;
import com.company.nfp.entity.enumerations.AccountingType;
import com.company.nfp.entity.enumerations.DocSubType;
import com.company.nfp.entity.enumerations.SourceGoodType;
import com.company.nfp.entity.mapped_super_class.TenantAttribute;
import io.jmix.core.DeletePolicy;
import io.jmix.core.entity.annotation.OnDelete;
import io.jmix.core.metamodel.annotation.Composition;
import io.jmix.core.metamodel.annotation.DependsOnProperties;
import io.jmix.core.metamodel.annotation.InstanceName;
import io.jmix.core.metamodel.annotation.JmixEntity;

import javax.persistence.*;
import javax.validation.constraints.Email;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

@JmixEntity
@Table(name = "NFP_GOOD", indexes = {
        @Index(name = "IDX_NFP_GOOD_COUNTERPARTY", columnList = "COUNTERPARTY_ID"),
        @Index(name = "IDX_NFP_GOOD_FIRM_ACCOUNT", columnList = "FIRM_ACCOUNT_ID"),
        @Index(name = "IDX_NFP_GOOD_SHOP", columnList = "SHOP_ID"),
        @Index(name = "IDX_NFP_GOOD_EMPLOYEE", columnList = "EMPLOYEE_ID"),
        @Index(name = "IDX_NFP_GOOD_STAT", columnList = "STAT_ID"),
        @Index(name = "IDX_NFP_GOOD_SERVICE", columnList = "SERVICE_ID")
})
@Entity(name = "nfp_Good")
public class Good extends TenantAttribute {
    @Column(name = "ACCOUNTING_TYPE")
    private String accountingType;

    @NotNull(message = "{msg://com.company.nfp.entity.source.money/Money.sourceMoneyType.validation.NotNull}")
    @Column(name = "SOURCE_GOOD_TYPE")
    private String sourceGoodType;

    @Column(name = "PN_IS_THERE")
    private Boolean pnIsThere;

    @JoinColumn(name = "COUNTERPARTY_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private Counterparty counterparty;

    @JoinColumn(name = "FIRM_ACCOUNT_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private FirmAccount firmAccount;

    @JoinColumn(name = "SHOP_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private Shop shop;

    @JoinColumn(name = "EMPLOYEE_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private Employee employee;

    @JoinColumn(name = "STAT_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private Stat stat;

    @JoinColumn(name = "SERVICE_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private Service service;

    @OnDelete(DeletePolicy.CASCADE)
    @Composition
    @OneToMany(mappedBy = "good")
    private List<GoodDetail> goodDetail;

    @Column(name = "NUM")
    private Long num;

    @Column(name = "NUMBER_")
    private String number;

    @Column(name = "DATE_")
    private LocalDateTime date;

    @Column(name = "REASON")
    private String reason;

    @Column(name = "FIRM_REPRESENTATIVE")
    private String firmRepresentative;

    @Column(name = "CUSTOMER_REPRESENTATIVE")
    private String customerRepresentative;

    @Column(name = "PLACE")
    private String place;

    //region Check records
    @Column(name = "DOC_SUB_TYPE")
    private String docSubType;

    @Column(name = "DOC_SUB_TYPE_STRING")
    private String docSubTypeString;

    @Column(name = "FN")
    private String fn;

    @Column(name = "QR")
    private String qr;

    @Column(name = "DFS_LINK")
    private String dfsLink;

    @Email(message = "{msg://com.company.nfp.entity.source.product/Good.email.validation.Email}")
    @Column(name = "EMAIL")
    private String email;

    @Column(name = "CHANGE_", precision = 19, scale = 2)
    private BigDecimal change;

    @Column(name = "BY_CASH", precision = 19, scale = 2)
    private BigDecimal byCash;

    @Column(name = "BY_CREDIT", precision = 19, scale = 2)
    private BigDecimal byCredit;

    @Column(name = "BY_CHECK", precision = 19, scale = 2)
    private BigDecimal byCheck;

    @Column(name = "BY_CARD", precision = 19, scale = 2)
    private BigDecimal byCard;

    @Column(name = "BY_BONUSES")
    private BigDecimal byBonuses;

    @Column(name = "CHECK_TEXT")
    private String checkText;

    //endregion
    @Column(name = "ORDER_DOC")
    private String orderDoc;

    @Column(name = "PAYER")
    private String payer;

    @Column(name = "TAX_INVOICE_NUMBER")
    private String taxInvoiceNumber;

    @Column(name = "TAX_INVOICE_DATE")
    private LocalDate taxInvoiceDate;

    @Column(name = "CONTRACT_NUMBER")
    private String contractNumber;

    @Column(name = "CONTRACT_DATE")
    private LocalDateTime contractDate;

    @Column(name = "SALE_CONDITION")
    private String saleCondition;

    @Column(name = "ACCEPTED")
    private String accepted;

    @Column(name = "PROXY")
    private String proxy;

    @Column(name = "EXTRA_CHARGE", precision = 19, scale = 2)
    private BigDecimal extraCharge;

    @Column(name = "SUM_PRICE", precision = 19, scale = 2)
    private BigDecimal sumPrice;

    @Column(name = "SUM_WITHOUT_TAX", precision = 19, scale = 2)
    private BigDecimal sumWithoutTax;

    @Column(name = "SUM_TAX", precision = 19, scale = 2)
    private BigDecimal sumTax;

    @Column(name = "SUM_", precision = 19, scale = 2)
    private BigDecimal sum;

    @Column(name = "BAL_WITHOUT_TAX", precision = 19, scale = 2)
    private BigDecimal balWithoutTax;

    @Column(name = "BAL_TAX", precision = 19, scale = 2)
    private BigDecimal balTax;

    @Column(name = "BAL", precision = 19, scale = 2)
    private BigDecimal bal;

    @Column(name = "DIFFERENCE", precision = 19, scale = 2)
    private BigDecimal difference;

    @Column(name = "SUM_IN_WORDS")
    private String sumInWords;

    @Column(name = "COMMENT_")
    @Lob
    private String comment;

    public Service getService() {
        return service;
    }

    public void setService(Service service) {
        this.service = service;
    }

    public String getPlace() {
        return place;
    }

    public void setPlace(String place) {
        this.place = place;
    }

    public String getCustomerRepresentative() {
        return customerRepresentative;
    }

    public void setCustomerRepresentative(String customerRepresentative) {
        this.customerRepresentative = customerRepresentative;
    }

    public String getFirmRepresentative() {
        return firmRepresentative;
    }

    public void setFirmRepresentative(String firmRepresentative) {
        this.firmRepresentative = firmRepresentative;
    }

    public BigDecimal getByCash() {
        return byCash;
    }

    public void setByCash(BigDecimal byCash) {
        this.byCash = byCash;
    }

    public String getReason() {
        return reason;
    }

    public void setReason(String reason) {
        this.reason = reason;
    }

    public Stat getStat() {
        return stat;
    }

    public void setStat(Stat stat) {
        this.stat = stat;
    }

    public AccountingType getAccountingType() {
        return accountingType == null ? null : AccountingType.fromId(accountingType);
    }

    public void setAccountingType(AccountingType accountingType) {
        this.accountingType = accountingType == null ? null : accountingType.getId();
    }

    public BigDecimal getBal() {
        return bal;
    }

    public void setBal(BigDecimal bal) {
        this.bal = bal;
    }

    public BigDecimal getBalTax() {
        return balTax;
    }

    public void setBalTax(BigDecimal balTax) {
        this.balTax = balTax;
    }

    public BigDecimal getBalWithoutTax() {
        return balWithoutTax;
    }

    public void setBalWithoutTax(BigDecimal balWithoutTax) {
        this.balWithoutTax = balWithoutTax;
    }

    public Boolean getPnIsThere() {
        return pnIsThere;
    }

    public void setPnIsThere(Boolean pnIsThere) {
        this.pnIsThere = pnIsThere;
    }

    public Long getNum() {
        return num;
    }

    public void setNum(Long num) {
        this.num = num;
    }

    public Employee getEmployee() {
        return employee;
    }

    public void setEmployee(Employee employee) {
        this.employee = employee;
    }

    public List<GoodDetail> getGoodDetail() {
        return goodDetail;
    }

    public void setGoodDetail(List<GoodDetail> goodDetail) {
        this.goodDetail = goodDetail;
    }

    public Shop getShop() {
        return shop;
    }

    public void setShop(Shop shop) {
        this.shop = shop;
    }

    public FirmAccount getFirmAccount() {
        return firmAccount;
    }

    public void setFirmAccount(FirmAccount firmAccount) {
        this.firmAccount = firmAccount;
    }

    public Counterparty getCounterparty() {
        return counterparty;
    }

    public void setCounterparty(Counterparty counterparty) {
        this.counterparty = counterparty;
    }

    public SourceGoodType getSourceGoodType() {
        return sourceGoodType == null ? null : SourceGoodType.fromId(sourceGoodType);
    }

    public void setSourceGoodType(SourceGoodType sourceGoodType) {
        this.sourceGoodType = sourceGoodType == null ? null : sourceGoodType.getId();
    }

    public BigDecimal getExtraCharge() {
        return extraCharge;
    }

    public void setExtraCharge(BigDecimal extraCharge) {
        this.extraCharge = extraCharge;
    }

    public String getProxy() {
        return proxy;
    }

    public void setProxy(String proxy) {
        this.proxy = proxy;
    }

    public String getAccepted() {
        return accepted;
    }

    public void setAccepted(String accepted) {
        this.accepted = accepted;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public String getSumInWords() {
        return sumInWords;
    }

    public void setSumInWords(String sumInWords) {
        this.sumInWords = sumInWords;
    }

    public BigDecimal getDifference() {
        return difference;
    }

    public void setDifference(BigDecimal difference) {
        this.difference = difference;
    }

    public BigDecimal getSumTax() {
        return sumTax;
    }

    public void setSumTax(BigDecimal sumTax) {
        this.sumTax = sumTax;
    }

    public BigDecimal getSumWithoutTax() {
        return sumWithoutTax;
    }

    public void setSumWithoutTax(BigDecimal sumWithoutTax) {
        this.sumWithoutTax = sumWithoutTax;
    }

    public BigDecimal getSumPrice() {
        return sumPrice;
    }

    public void setSumPrice(BigDecimal sumPrice) {
        this.sumPrice = sumPrice;
    }

    public String getSaleCondition() {
        return saleCondition;
    }

    public void setSaleCondition(String saleCondition) {
        this.saleCondition = saleCondition;
    }

    public LocalDateTime getContractDate() {
        return contractDate;
    }

    public void setContractDate(LocalDateTime contractDate) {
        this.contractDate = contractDate;
    }

    public String getContractNumber() {
        return contractNumber;
    }

    public void setContractNumber(String contractNumber) {
        this.contractNumber = contractNumber;
    }

    public LocalDate getTaxInvoiceDate() {
        return taxInvoiceDate;
    }

    public void setTaxInvoiceDate(LocalDate taxInvoiceDate) {
        this.taxInvoiceDate = taxInvoiceDate;
    }

    public String getTaxInvoiceNumber() {
        return taxInvoiceNumber;
    }

    public void setTaxInvoiceNumber(String taxInvoiceNumber) {
        this.taxInvoiceNumber = taxInvoiceNumber;
    }

    public String getPayer() {
        return payer;
    }

    public void setPayer(String payer) {
        this.payer = payer;
    }

    public String getOrderDoc() {
        return orderDoc;
    }

    public void setOrderDoc(String orderDoc) {
        this.orderDoc = orderDoc;
    }

    public void setByBonuses(BigDecimal byBonuses) {
        this.byBonuses = byBonuses;
    }

    public BigDecimal getByBonuses() {
        return byBonuses;
    }

    public String getCheckText() {
        return checkText;
    }

    public void setCheckText(String checkText) {
        this.checkText = checkText;
    }

    public BigDecimal getByCard() {
        return byCard;
    }

    public void setByCard(BigDecimal byCard) {
        this.byCard = byCard;
    }

    public BigDecimal getByCheck() {
        return byCheck;
    }

    public void setByCheck(BigDecimal byCheck) {
        this.byCheck = byCheck;
    }

    public BigDecimal getByCredit() {
        return byCredit;
    }

    public void setByCredit(BigDecimal byCredit) {
        this.byCredit = byCredit;
    }

    public BigDecimal getChange() {
        return change;
    }

    public void setChange(BigDecimal change) {
        this.change = change;
    }

    public BigDecimal getSum() {
        return sum;
    }

    public void setSum(BigDecimal sum) {
        this.sum = sum;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getDfsLink() {
        return dfsLink;
    }

    public void setDfsLink(String dfsLink) {
        this.dfsLink = dfsLink;
    }

    public String getQr() {
        return qr;
    }

    public void setQr(String qr) {
        this.qr = qr;
    }

    public String getFn() {
        return fn;
    }

    public void setFn(String fn) {
        this.fn = fn;
    }

    public String getDocSubTypeString() {
        return docSubTypeString;
    }

    public void setDocSubTypeString(String docSubTypeString) {
        this.docSubTypeString = docSubTypeString;
    }

    public DocSubType getDocSubType() {
        return docSubType == null ? null : DocSubType.fromId(docSubType);
    }

    public void setDocSubType(DocSubType docSubType) {
        this.docSubType = docSubType == null ? null : docSubType.getId();
    }

    public LocalDateTime getDate() {
        return date;
    }

    public void setDate(LocalDateTime date) {
        this.date = date;
    }

    public String getNumber() {
        return number;
    }

    public void setNumber(String number) {
        this.number = number;
    }

    @InstanceName
    @DependsOnProperties({"number", "date"})
    public String getInstanceName() {
        return String.format("%s :: %s", number, date);
    }
}