package com.company.nfp.entity.source.revaluation;

import com.company.nfp.entity.dictionaries.firm.Firm;
import com.company.nfp.entity.dictionaries.product.Product;
import com.company.nfp.entity.enumerations.SourceRevaluationType;
import com.company.nfp.entity.mapped_super_class.TenantAttribute;
import io.jmix.core.metamodel.annotation.JmixEntity;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDateTime;

@JmixEntity
@Table(name = "NFP_REVALUATION", indexes = {
        @Index(name = "IDX_NFP_REVALUATION_PRODUCT", columnList = "PRODUCT_ID"),
        @Index(name = "IDX_NFP_REVALUATION_FIRM", columnList = "FIRM_ID")
})
@Entity(name = "nfp_Revaluation")
public class Revaluation extends TenantAttribute {
    @Column(name = "SOURCE_REVALUATION_TYPE", nullable = false)
    @NotNull
    private String sourceRevaluationType;

    @JoinColumn(name = "FIRM_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    private Firm firm;

    @JoinColumn(name = "PRODUCT_ID", nullable = false)
    @NotNull
    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    private Product product;

    @Column(name = "DATE_", nullable = false)
    @NotNull
    private LocalDateTime date;

    @Column(name = "COST_PRICE", nullable = false, precision = 19, scale = 2)
    @NotNull
    private BigDecimal costPrice;

    @Column(name = "OLD_COST", nullable = false, precision = 19, scale = 2)
    @NotNull
    private BigDecimal oldCost;

    @Column(name = "COST", nullable = false, precision = 19, scale = 2)
    @NotNull
    private BigDecimal cost;

    public Firm getFirm() {
        return firm;
    }

    public void setFirm(Firm firm) {
        this.firm = firm;
    }

    public BigDecimal getCostPrice() {
        return costPrice;
    }

    public void setCostPrice(BigDecimal costPrice) {
        this.costPrice = costPrice;
    }

    public BigDecimal getCost() {
        return cost;
    }

    public void setCost(BigDecimal cost) {
        this.cost = cost;
    }

    public BigDecimal getOldCost() {
        return oldCost;
    }

    public void setOldCost(BigDecimal oldCost) {
        this.oldCost = oldCost;
    }

    public LocalDateTime getDate() {
        return date;
    }

    public void setDate(LocalDateTime date) {
        this.date = date;
    }

    public SourceRevaluationType getSourceRevaluationType() {
        return sourceRevaluationType == null ? null : SourceRevaluationType.fromId(sourceRevaluationType);
    }

    public void setSourceRevaluationType(SourceRevaluationType sourceRevaluationType) {
        this.sourceRevaluationType = sourceRevaluationType == null ? null : sourceRevaluationType.getId();
    }

    public Product getProduct() {
        return product;
    }

    public void setProduct(Product product) {
        this.product = product;
    }
}