package com.company.nfp.screen.prro.cashier_check_start;

import com.company.nfp.app.prro.check_prro_status.CheckPrroStatusService;
import com.company.nfp.app.prro.get_check_by_fiscal_number.GetCheckByFiscalNumberService;
import com.company.nfp.app.prro.x_report.XReportService;
import com.company.nfp.entity.User;
import com.company.nfp.entity.enumerations.DocSubType;
import com.company.nfp.entity.not_persistent.prro.ResultCheckPrroStatusEntity;
import com.company.nfp.screen.prro.check.check_screen.check.CheckScreen;
import com.company.nfp.screen.prro.service_in_out.ServiceInOutScreen;
import com.company.nfp.screen.prro.xzreport.xreport.XreportScreen;
import io.jmix.core.Messages;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.ui.Notifications;
import io.jmix.ui.Screens;
import io.jmix.ui.action.Action;
import io.jmix.ui.component.TextField;
import io.jmix.ui.component.TextInputField;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.net.URISyntaxException;

@UiController("nfp_CashierCheckStartScreen")
@UiDescriptor("cashier-check-start-screen.xml")
public class CashierCheckStartScreen extends Screen {
    //region Injections
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private Screens screens;
    @Autowired
    private CheckPrroStatusService checkPrroStatusService;
    @Autowired
    private Messages messages;
    @Autowired
    private Notifications notifications;
    @Autowired
    private MessageBundle messageBundle;
    @Autowired
    private TextField fiscalNumberField;
    @Autowired
    private GetCheckByFiscalNumberService getCheckByFiscalNumberService;
    @Autowired
    private XReportService xReportService;
    //endregion

    //region Subscriptions
/*
    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        user = getUser();
    }
*/

    @Subscribe("businessCashDepositAction")
    public void onBusinessCashDepositAction(Action.ActionPerformedEvent event) {
        var serviceInOutScreen = screens.create(ServiceInOutScreen.class);
        serviceInOutScreen.setDocSubType(DocSubType.CASH_DEPOSIT);
        serviceInOutScreen.setUserId(getUser().getId());
        screens.show(serviceInOutScreen);
    }

    @Subscribe("businessCashDropAction")
    public void onBusinessCashDropAction(Action.ActionPerformedEvent event) throws IOException, InterruptedException {
        //if (checkShiftStatus()) return;
        var serviceInOutScreen = screens.create(ServiceInOutScreen.class);
        serviceInOutScreen.setDocSubType(DocSubType.CASH_DROP);
        serviceInOutScreen.setUserId(getUser().getId());
        screens.show(serviceInOutScreen);

    }

    @Subscribe("salesRegistrationAction")
    public void onSalesRegistrationAction(Action.ActionPerformedEvent event) throws IOException, InterruptedException {
        //if (checkShiftStatus()) return;
        var screen = screens.create(CheckScreen.class);
        screen.setDocSubType(DocSubType.TEXT);
        screen.setUserId(getUser().getId());
        screens.show(screen);
    }

    @Subscribe("goodsReturnAction")
    public void onGoodsReturnAction(Action.ActionPerformedEvent event) throws IOException, InterruptedException {
        if (checkShiftStatus()) return;
        fiscalNumberField.setVisible(true);
        fiscalNumberField.setInputPrompt(messages.getMessage("msg://com.company.nfp.screen.prro.cashier_check_start/inputPrompt.fiscalNumber"));
        fiscalNumberField.focus();
    }

    @Subscribe("fiscalNumberField")
    public void onFiscalNumberFieldEnterPress(TextInputField.EnterPressEvent event) {
        if (fiscalNumberField.getValue() != null) {
            if (getCheckByFiscalNumberService.getCheck(fiscalNumberField.getRawValue())) {
                fiscalNumberField.setVisible(false);
                var screen = screens.create(CheckScreen.class);
                screen.setDocSubType(DocSubType.RECEIPT_POSITION);
                screen.setFn(fiscalNumberField.getRawValue());
                screen.show();
            } else {
                notificationShow(
                        messages.getMessage("msg:///notification.error.noCheckFound"),
                        Notifications.NotificationType.WARNING
                );
                fiscalNumberField.selectAll();
            }
        }
    }

    @Subscribe("xReportAction")
    public void onXReportAction(Action.ActionPerformedEvent event) throws IOException, InterruptedException, URISyntaxException {
        if (checkShiftStatus()) return;
        reportActionExecute("innovate/xreport");
    }

    @Subscribe("zReportAction")
    public void onZReportAction(Action.ActionPerformedEvent event) throws URISyntaxException, IOException, InterruptedException {
        reportActionExecute("innovate/zreport");
    }

    @Subscribe("checkPrroStatusAction")
    public void onCheckPrroStatusAction(Action.ActionPerformedEvent event) throws IOException, InterruptedException {
        var shiftStatus = checkShiftStatus();
        if (shiftStatus)
            notificationShow(messages.getMessage("msg:///notification.info.shiftStatusOk"),
                    Notifications.NotificationType.TRAY);
    }
    //endregion

    //region Methods
    /**
     * Check the shift status
     *
     * @return TRUE if the shift closed, FALSE if the shift opened
     */
    private boolean checkShiftStatus() throws IOException, InterruptedException {
        var prroStatus = checkPrroStatusService.checkPrroStatus(getUser().getId());
        showResultCheckPrroStatus(prroStatus);
        if (!prroStatus.getShiftOpen()) return true;
        return false;

    }

    /**
     * Show message according the shift status
     *
     * @param result check Prro Status Entity
     * @see ResultCheckPrroStatusEntity
     */
    private void showResultCheckPrroStatus(ResultCheckPrroStatusEntity result) {
        if (result.getDurationToMinutes() != null && result.getMessage() != null) {
            notificationShow(messageBundle.formatMessage(result.getMessage(), result.getDurationToMinutes()),
                    Notifications.NotificationType.WARNING);
        } else if (result.getMessage() != null) {
            notificationShow(messages.getMessage(result.getMessage()),
                    Notifications.NotificationType.ERROR);
        }
    }

    private void reportActionExecute(String path) throws URISyntaxException, IOException, InterruptedException {
        var text = xReportService.executeRequest(getUser().getId(), path);
        if ("Error".equals(text)) {
            notificationShow(
                    messages.getMessage("msg:///notifications.warning.invalidOperation"),
                    Notifications.NotificationType.WARNING
            );
        } else {
            notificationShow(
                    messages.getMessage("msg:///notifications.success.successOperation"),
                    Notifications.NotificationType.TRAY
            );
            var xzReportScreen = screens.create(XreportScreen.class);
            xzReportScreen.setText(text);
            screens.show(xzReportScreen);
        }
    }

    public User getUser() {
        return (User) currentAuthentication.getUser();
    }

    /**
     * Show notification
     *
     * @param key              Message Bandle Key
     * @param notificationType Notification Type
     */
    private void notificationShow(String key, Notifications.NotificationType notificationType) {
        String message = messages.getMessage(key);
        notifications.create(notificationType)
                .withDescription(message)
                .show();
    }
    //endregion

}