package com.company.nfp.screen.prro.check.check_screen.check;

import com.company.nfp.app.calculation.product_calculations.CalcProductCostPerDateService;
import com.company.nfp.app.calculation.product_calculations.CalcProductLastCostBySaleService;
import com.company.nfp.app.calculation.product_calculations.ProductQuantityBalanceService;
import com.company.nfp.app.get_entity.get_current_prro.GetCurrentPrroService;
import com.company.nfp.app.get_entity.get_document_props.GetDocumentPropsService;
import com.company.nfp.app.get_entity.get_product_by_barcode.GetProductByBarcodeService;
import com.company.nfp.app.get_entity.get_sales_method.GetSalesMethodService;
import com.company.nfp.app.prro.helpers.GetCheckDfsLinkService;
import com.company.nfp.app.prro.helpers.GetPrroByUserService;
import com.company.nfp.app.prro.receipt.SaleReceiptService;
import com.company.nfp.app.prro.send_email.SendEmailService;
import com.company.nfp.app.set_next_number.SetNextNumberService;
import com.company.nfp.entity.User;
import com.company.nfp.entity.dictionaries.product.Product;
import com.company.nfp.entity.enumerations.DocSubType;
import com.company.nfp.entity.enumerations.SalesMethod;
import com.company.nfp.entity.enumerations.SourceGoodType;
import com.company.nfp.entity.not_persistent.prro.SaleReturnReceiptEntity;
import com.company.nfp.entity.source.product.Good;
import com.company.nfp.entity.source.product.GoodDetail;
import com.company.nfp.entity.source.product.GoodDetailExcise;
import com.company.nfp.screen.prro.check.check_view.checkview.CheckViewScreen;
import com.company.nfp.screen.prro.check.excise.excise.ExciseScreen;
import io.jmix.core.DataManager;
import io.jmix.core.Messages;
import io.jmix.core.SaveContext;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.ui.Notifications;
import io.jmix.ui.Screens;
import io.jmix.ui.action.Action;
import io.jmix.ui.component.*;
import io.jmix.ui.download.DownloadFormat;
import io.jmix.ui.download.Downloader;
import io.jmix.ui.model.CollectionLoader;
import io.jmix.ui.model.CollectionPropertyContainer;
import io.jmix.ui.model.InstanceContainer;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.math.BigDecimal;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

@UiController("nfp_CheckScreen")
@UiDescriptor("check-screen.xml")
public class CheckScreen extends Screen {
    //region Properties
    private DocSubType docSubType;
    private final List<GoodDetail> checkDetails = new ArrayList<>();
    private String salesMethod;
    private String checkString;
    private String fn;
    //endregion

    //region Injections
    @Autowired
    private CollectionLoader<Product> productsDl;
    @Autowired
    private InstanceContainer<Good> checkDc;
    @Autowired
    private InstanceContainer<GoodDetail> checkDetailOneDc;
    @Autowired
    private DataManager dataManager;
    @Autowired
    private Messages messages;
    @Autowired
    private Notifications notifications;
    @Autowired
    private SetNextNumberService setNextNumberService;
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private CollectionPropertyContainer<GoodDetail> checkDetailDc;
    @Autowired
    private GetDocumentPropsService getDocumentPropsService;
    @Autowired
    private TextField<String> barcodeField;
    @Autowired
    private GetSalesMethodService getSalesMethodService;
    @Autowired
    private TextField<BigDecimal> costField;
    @Autowired
    private GetProductByBarcodeService getProductByBarcodeService;
    @Autowired
    private ProductQuantityBalanceService productQuantityBalanceService;
    @Autowired
    private TextField<BigDecimal> balanceField;
    @Autowired
    private CalcProductCostPerDateService calcProductCostPerDateService;
    @Autowired
    private TextField<BigDecimal> quantityField;
    @Autowired
    private CalcProductLastCostBySaleService calcProductLastCostBySaleService;
    @Autowired
    private EntityComboBox<Product> productField;
    @Autowired
    private Screens screens;
    @Autowired
    private TextField<BigDecimal> byCashField;
    @Autowired
    private TextField<BigDecimal> byCardField;
    @Autowired
    private Button turnTypePayBtn;
    @Autowired
    private TextField<BigDecimal> byCheckField;
    @Autowired
    private TextField<BigDecimal> byCreditField;
    @Autowired
    private Button commitBtn;
    @Autowired
    private CheckBox itFiscalizationCheckBox;
    @Autowired
    private CheckBox itSendCheckBox;
    @Autowired
    private EntityPicker<String> emailLookupField;
    @Autowired
    private SaleReceiptService saleReceiptService;
    @Autowired
    private GetCheckDfsLinkService getCheckDfsLinkService;
    @Autowired
    private CheckBox itPrintCheckBox;
    @Autowired
    private GetPrroByUserService getPrroByUserService;
    @Autowired
    private Downloader downloader;
    @Autowired
    private SendEmailService sendEmailService;
    @Autowired
    private GetCurrentPrroService getCurrentPrroService;
    //endregion

    //region Setters
    public void setDocSubType(DocSubType docSubType) {
        this.docSubType = docSubType;
    }

    public void setUserId(UUID userId) {
    }

/*
    public void setSalesMethod(String salesMethod) {
        this.salesMethod = salesMethod;
    }
*/
    //endregion

    //region Subscriptions
    @Subscribe
    public void onBeforeShow(BeforeShowEvent event) {
        newCheckCreate();
        newCheckDetailCreate();
        productsDl.load();
    }

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        // Get the current sales method for the shop
        this.salesMethod = getSalesMethodService.getSalesMethod(getUser().getId());
        if (this.salesMethod != null) {
            if (Objects.equals(this.salesMethod, SalesMethod.SALE.getId())) costField.setEditable(true);
        }
    }

    @Subscribe("barcodeField")
    public void onBarcodeFieldEnterPress(TextInputField.EnterPressEvent event) {
        var product = getProductByBarcodeService.getProductByBarcode(barcodeField.getRawValue());
        if (product != null) {
            setProductCosts(product);
            checkDetailOneDc.getItem().setProduct(product);
        } else {
            notificationShow(
                    messages.getMessage("msg:///notifications.warning.productNotFound"),
                    Notifications.NotificationType.WARNING
            );
            clearDetail();
            newCheckDetailCreate();
        }
    }

    @Subscribe("productField")
    public void onProductFieldValueChange(HasValue.ValueChangeEvent<Product> event) {
        if (productField.getValue() != null) setProductCosts(productField.getValue());

    }

    @Subscribe("quantityField")
    public void onQuantityFieldEnterPress(TextInputField.EnterPressEvent event) {
        if (saleQuantityCheck()) {

            if (checkDetailOneDc.getItem().getProduct().getIsExcise()) {
                var screen = screens.create(ExciseScreen.class);
                screen.setCheckDetail(checkDetailOneDc.getItem());
                screen.setQuantity(checkDetailOneDc.getItem().getQuantity());
                screen.addAfterCloseListener(afterCloseEvent -> detailPrepare());
                screen.show();
            } else detailPrepare();
        }

    }

    @Subscribe("turnAction")
    public void onTurnAction(Action.ActionPerformedEvent event) {
        if (Objects.requireNonNull(byCashField.getValue()).compareTo(BigDecimal.ZERO) == 0) {
            byCashField.setValue(byCardField.getValue());
            byCardField.setValue(BigDecimal.ZERO);
            turnTypePayBtn.setCaption(messages.getMessage("msg://com.company.nfp.screen.prro.check.check_screen.check/checkScreen.turn.card"));
            calcDetailAmounts();
            changeCalc();

        } else if (Objects.requireNonNull(byCardField.getValue()).compareTo(BigDecimal.ZERO) == 0) {
            byCardField.setValue(byCashField.getValue());
            byCashField.setValue(BigDecimal.ZERO);
            turnTypePayBtn.setCaption(messages.getMessage("msg://com.company.nfp.screen.prro.check.check_screen.check/checkScreen.turn.cash"));
            calcDetailAmounts();
            changeCalc();

        }
    }

    /*
    @Subscribe("itSendCheckBox")
    public void onItSendCheckBoxValueChange(HasValue.ValueChangeEvent<Boolean> event) {
        emailLookupField.isVisible = itSendCheckBox.value == true

    }
*/

    @Subscribe("byCashField")
    public void onByCashFieldEnterPress(TextInputField.EnterPressEvent event) {
        changeCalc();
        byCardField.focus();
        byCardField.selectAll();
    }

    @Subscribe("byCardField")
    public void onByCardFieldEnterPress(TextInputField.EnterPressEvent event) {
        changeCalc();
        byCheckField.focus();
        byCheckField.selectAll();
    }

    @Subscribe("byCheckField")
    public void onByCheckFieldEnterPress(TextInputField.EnterPressEvent event) {
        changeCalc();
        byCreditField.focus();
        byCreditField.selectAll();
    }

    @Subscribe("byCreditField")
    public void onByCreditFieldEnterPress(TextInputField.EnterPressEvent event) {
        changeCalc();
        commitBtn.focus();
    }

    @Subscribe("commitAction")
    public void onCommitAction(Action.ActionPerformedEvent event) throws IOException, InterruptedException {
        if (!itFiscalizationCheckBox.isChecked() && docSubType != DocSubType.RECEIPT_POSITION)
            docSubType = DocSubType.NON_FISCAL_RECEIPT;
        if (Boolean.TRUE.equals(itSendCheckBox.getValue())) {
            if (!emailLookupField.isValid() || emailLookupField.getValue() == null) {
                notificationShow(messages.getMessage("msg:///notification.error.wrongEmailFormat"),
                        Notifications.NotificationType.ERROR);
                return;
            }
        }
        SaleReturnReceiptEntity result;
        switch (docSubType) {
            case TEXT:          // Check to fiscalization creating
                if (!checkCommit()) return;
                result = saleReceiptService.executeRequest(docSubType, checkDc.getItem().getId());
                if (checkCheck(result)) return;
                checkPrint();
                if (Boolean.TRUE.equals(itSendCheckBox.getValue())) {
                    var prro = getCurrentPrroService.getCurrentPrro(getUser().getId());
                    var pathAttachment = prro.getPathReceipt();
                    Boolean sentResult =
                            sendEmailService.sendByEmail(emailLookupField.getValue(), "testCaption", pathAttachment);
                    if (sentResult) {
                        notificationShow(
                                messages.getMessage("msg:///notification.info.checkIsSentToEmail"),
                                Notifications.NotificationType.TRAY
                        );
                    } else {
                        notificationShow(
                                messages.getMessage("msg:///notification.error.checkIsSentToEmail"),
                                Notifications.NotificationType.ERROR
                        );
                    }
                }

                newCheckCreate();
                newCheckDetailCreate();
                itFiscalizationCheckBox.setValue(false);

                break;
            case RECEIPT_POSITION:          // The Return Check creating
                if (!checkCommit()) return;
                if (fn != null) {
                    result = saleReceiptService.executeRequest(docSubType, checkDc.getItem().getId(), fn);
                    if (checkCheck(result)) return;
                    checkPrint();
                }
                break;
            case NON_FISCAL_RECEIPT:
                if (!checkCommit()) return;
                docSubType = DocSubType.NON_FISCAL_RECEIPT;
                result = saleReceiptService.executeRequest(docSubType, checkDc.getItem().getId());
                if (result.getStatus() == 200) {
                    checkPrint();
                    notificationShow(
                            messages.getMessage("msg:///notifications.success.successOperation"),
                            Notifications.NotificationType.TRAY
                    );
                } else {
                    notificationShow(
                            messages.getMessage("msg:///notifications.warning.invalidOperation"),
                            Notifications.NotificationType.WARNING
                    );
                }

                newCheckCreate();
                newCheckDetailCreate();
                break;
        }
    }

    @Subscribe("cancelAction")
    public void onCancelAction(Action.ActionPerformedEvent event) {
        close(WINDOW_CLOSE_ACTION);
    }
    //endregion

    //region Methods
    private void checkPrint() {
        if (Boolean.TRUE.equals(itPrintCheckBox.getValue())) {
            try {
                var prro = getPrroByUserService.getPrro(getUser().getId());
                if (prro.getPathReceipt() != null) {
                    var pdfPath = Paths.get(prro.getPathReceipt());
                    var pdf = Files.readAllBytes(pdfPath);
                    downloader.download(pdf, "receipt.pdf", DownloadFormat.PDF);
                    //exportDisplay.show(new ByteArrayDataProvider(pdf), "receipt.pdf", ExcelExporter.ExportFormat.PDF)
                }
            } catch (Exception e) {
                notificationShow(
                        messages.getMessage("msg:///notification.error.checkPrintError"),
                        Notifications.NotificationType.ERROR
                );
            }
        }

    }

    private boolean checkCheck(SaleReturnReceiptEntity result) {
        if (result.getStatus() == 0) {
            notificationShow(
                    messages.getMessage("msg:///notification.error.noCheckFound"),
                    Notifications.NotificationType.ERROR
            );
            return true;
        }
        if (result.getFiscalNumber() != null) {
            checkString = getCheckDfsLinkService.getLink(result.getQR()); //result.QR?.let { getCheckDfsLinkService.getLink(it) }
            this.notificationShow(
                    messages.getMessage("msg:///notification.info.checkFiscalized"),
                    Notifications.NotificationType.TRAY
            );
            qrCodeView();
            return false;
        } else {
            this.notificationShow(
                    messages.getMessage("msg:///notification.error.checkFiscalized"),
                    Notifications.NotificationType.ERROR
            );
            return false;
        }
    }

    private void qrCodeView() {
        var screen = screens.create(CheckViewScreen.class);
        screen.setCheckString(checkString);
        screens.show(screen);

    }

    private boolean checkCommit() {
        var result = false;
        if (checkDetailDc.getItems().size() == 0) {
            notificationShow(messages.getMessage("msg:///notification.error.noProductsInCheck"),
                    Notifications.NotificationType.ERROR);
            return result;
        }
        changeCalc();

        var number = setNextNumberService.setDocumentNumber(SourceGoodType.ZO_CHECK, getUser().getId());
        checkDc.getItem().setNumber(String.valueOf(number));
        checkDc.getItem().setNum(number);

        SaveContext saveContext = new SaveContext().setDiscardSaved(true);
        saveContext.saving(checkDc.getItem());
        for (GoodDetail detail : checkDetailDc.getItems()) {
            saveContext.saving(detail);
            for (GoodDetailExcise excise : detail.getExcise()) {
                saveContext.saving(excise);
            }
        }
        dataManager.save(saveContext);
        result = true;
        return result;
    }

    private void changeCalc() {
        var paid = checkDc.getItem().getByCash().add(checkDc.getItem().getByCard())
                .add(checkDc.getItem().getByCheck()).add(checkDc.getItem().getByCredit())
                .add(checkDc.getItem().getByBonuses());
        checkDc.getItem().setChange(paid.subtract(checkDc.getItem().getSum()));
    }

    private void clearDetail() {
        checkDetailOneDc.setItem(dataManager.create(GoodDetail.class));
    }

    private void setProductCosts(Product product) {
        var productId = product.getId();
        var date = checkDc.getItem().getDate();
        switch (this.salesMethod) {
            case "Accounting":
                if (docSubType == DocSubType.TEXT) calcProductBalance(product);
                if (productId != null) {
                    var costsPerDate = calcProductCostPerDateService.calcProductCostPerDate(getUser().getId(),
                            productId, date);
                    checkDetailOneDc.getItem().setCostPrice(costsPerDate.getCostPrice());
                    checkDetailOneDc.getItem().setCost(costsPerDate.getCost());
                    quantityField.focus();
                    quantityField.setCursorPosition(0);
                    quantityField.selectAll();
                }
                break;
            case "Sale":

                if (productId != null) {
                    var cost = calcProductLastCostBySaleService.calcCostsPerDate(productId, checkDc.getItem().getDate());
                    checkDetailOneDc.getItem().setCostPrice(BigDecimal.ZERO);
                    if (cost != null) {
                        checkDetailOneDc.getItem().setCost(cost);
                        quantityField.focus();
                        quantityField.setCursorPosition(0);
                        quantityField.selectAll();
                    } else {
                        checkDetailOneDc.getItem().setCost(BigDecimal.ZERO);
                        costField.focus();
                        costField.setCursorPosition(0);
                        costField.selectAll();
                        //no (messageBundle.getMessage("notifications.warning.productNoPurchased"), Notifications.NotificationType.WARNING)
                    }
                    break;
                }
        }

    }

    private void calcProductBalance(Product product) {
        var productId = product.getId();
        var date = checkDc.getItem().getDate();
        var balance = productQuantityBalanceService.productQuantityBalance(productId, date);
        for (GoodDetail detail : checkDetailDc.getItems()) {
            if (detail.getProduct() == product) {
                if (balance != null) {
                    balance = balance.subtract(detail.getQuantity());
                }
            }
        }
        balanceField.setValue(balance);
    }

    private void newCheckDetailCreate() {
        var checkDetail = dataManager.create(GoodDetail.class);
        var check = checkDc.getItem();
        checkDetail.setGood(check);
        checkDetail.setQuantity(BigDecimal.ONE);
        checkDetailOneDc.setItem(checkDetail);
        barcodeField.focus();
    }

    private void newCheckCreate() {
        checkDetails.removeAll(checkDetails);
        checkDc.setItem(null);
        checkDetailOneDc.setItem(null);
        checkDetailDc.setItems(null);

        var check = dataManager.create(Good.class);
        var docProps = getDocumentPropsService.getDocumentProps(getUser().getId());
        if (docProps == null) {
            notificationShow("msg://com.company.nfp.notification/invalidOperation", Notifications.NotificationType.WARNING);
        } else {
            check.setEmployee(docProps.getEmployee());
            check.setShop(docProps.getShop());
            check.setFirmAccount(docProps.getAccount());
            check.setDate(LocalDateTime.now());
            check.setSum(BigDecimal.ZERO);
            check.setChange(BigDecimal.ZERO);
            check.setByCash(BigDecimal.ZERO);
            check.setByCard(BigDecimal.ZERO);
            check.setByCheck(BigDecimal.ZERO);
            check.setByCredit(BigDecimal.ZERO);
            check.setByBonuses(BigDecimal.ZERO);
            checkDc.setItem(check);
        }
    }

    private void detailPrepare() {
        calcDetailAmounts();
        checkDetails.add(checkDetailOneDc.getItem());
        checkDetailDc.setItems(checkDetails);
        calcCheckAmounts();
        clearDetail();
        newCheckDetailCreate();
    }

    private void calcCheckAmounts() {
        var num = 1;
        var sumPrice = BigDecimal.ZERO;
        var sum = BigDecimal.ZERO;

        for (GoodDetail checkDetail : checkDetailDc.getItems()) {
            checkDetail.setNumberInOrder(num++);
            sumPrice = sumPrice.add(checkDetail.getSumPrice());
            sum = sum.add(checkDetail.getSum());
        }
        checkDc.getItem().setSumPrice(sumPrice);
        checkDc.getItem().setSum(sum);
        checkDc.getItem().setDifference(sum.subtract(sumPrice));
        //checkDc.getItem().paid = sum
        checkDc.getItem().setByCash(sum);
    }

    private void calcDetailAmounts() {
        checkDetailOneDc.getItem().setSumPrice(checkDetailOneDc.getItem().getCostPrice().multiply(checkDetailOneDc.getItem().getQuantity()));
        checkDetailOneDc.getItem().setSum(checkDetailOneDc.getItem().getCost().multiply(checkDetailOneDc.getItem().getQuantity()));
    }

    private boolean saleQuantityCheck() {
        var result = false;
        switch (this.salesMethod) {
            case "Accounting":
                if (this.docSubType == DocSubType.TEXT) {
                    if (balanceField.getValue() == null ||
                            balanceField.getValue().compareTo(quantityField.getValue()) < 0 ||
                            quantityField.getValue().compareTo(BigDecimal.ZERO) < 0) {
                        notificationShow(messages.getMessage("msg:///notifications.warning.negativeBalance"),
                                Notifications.NotificationType.WARNING);
                    } else result = true;
                }
                break;
            case "Sale":
                result = true;
                break;
        }
        return result;
    }

    private User getUser() {
        User user;
        try {
            user = (User) currentAuthentication.getUser();
            return user;
        } catch (Exception e) {
            notificationShow("invalid current user", Notifications.NotificationType.ERROR);
            return null;
        }
    }

    /**
     * Show notification
     *
     * @param key              Message Bundle Key
     * @param notificationType Notification Type
     */
    private void notificationShow(String key, Notifications.NotificationType notificationType) {
        String message = messages.getMessage(key);
        notifications.create(notificationType)
                .withDescription(message)
                .show();
    }

    public void setFn(String fn) {
        this.fn = fn;
    }
    //endregion
}
