package com.company.nfp.screen.prro.service_in_out;

import com.company.nfp.app.prro.in_out_cash.InOutCashService;
import com.company.nfp.entity.enumerations.DocSubType;
import io.jmix.core.Messages;
import io.jmix.ui.Notifications;
import io.jmix.ui.action.Action;
import io.jmix.ui.component.TextField;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.util.UUID;

@UiController("nfp_ServiceInOutScreen")
@UiDescriptor("service-in-out-screen.xml")
public class ServiceInOutScreen extends Screen {

    private DocSubType docSubType;
    private UUID userId;
    @Autowired
    private MessageBundle messageBundle;
    @Autowired
    private TextField<BigDecimal> sumField;
    @Autowired
    private InOutCashService inOutCashService;
    @Autowired
    private Messages messages;
    @Autowired
    private Notifications notifications;

    public void setDocSubType(DocSubType docSubType) {
        this.docSubType = docSubType;
    }

    public void setUserId(UUID userId) {
        this.userId = userId;
    }

    /**
     * Customize Screen Header
     *
     * @param event Before Show Event
     */
    @Subscribe
    public void onBeforeShow(BeforeShowEvent event) {
        switch (docSubType) {
            case CASH_DEPOSIT:
                event.getSource().getWindow().setCaption(messageBundle.getMessage("window.caption.serviceIn"));
                break;
            case CASH_DROP:
                event.getSource().getWindow().setCaption(messageBundle.getMessage("window.caption.serviceOut"));
                break;
        }
    }


    @Subscribe("executeAction")
    public void onExecuteAction(Action.ActionPerformedEvent event) {
        var sum = sumField.getValue();
        if (sum != null && sum.compareTo(BigDecimal.ZERO) >= 0) {
            Integer result = inOutCashService.executeRequest(docSubType, sumField.getValue(), userId);
            switch (result) {
                case 200:
                    notificationShow("com.company.nfp.notification/successful", Notifications.NotificationType.TRAY);
                    break;
                default:
                    notificationShow("com.company.nfp.notification/invalidOperation", Notifications.NotificationType.WARNING);
                    break;
            }
        } else {
            notificationShow("com.company.nfp.notification/invalidOperation", Notifications.NotificationType.WARNING);
        }
    }

    @Subscribe("cancelAction")
    public void onCancelAction(Action.ActionPerformedEvent event) {
        close(WINDOW_CLOSE_ACTION);
    }

    /**
     * Show notification
     *
     * @param key              Message Bandle Key
     * @param notificationType Notification Type
     */
    private void notificationShow(String key, Notifications.NotificationType notificationType) {
        String message = messages.getMessage(key);
        notifications.create(notificationType)
                .withDescription(message)
                .show();
    }
}