package com.company.nfp.screen.services.koatuu_import;

import com.company.nfp.app.import_data.from_dbf.ImportFromDbfService;
import com.company.nfp.app.import_data.from_excel.ImportFromExcelService;
import com.google.common.collect.Lists;
import io.jmix.core.Messages;
import io.jmix.ui.Notifications;
import io.jmix.ui.Notifications.NotificationType;
import io.jmix.ui.component.Button.ClickEvent;
import io.jmix.ui.component.ComboBox;
import io.jmix.ui.component.FileStorageUploadField;
import io.jmix.ui.component.SingleFileUploadField.FileUploadSucceedEvent;
import io.jmix.ui.screen.Screen;
import io.jmix.ui.screen.StandardOutcome;
import io.jmix.ui.screen.Subscribe;
import io.jmix.ui.screen.UiController;
import io.jmix.ui.screen.UiDescriptor;
import io.jmix.ui.upload.TemporaryStorage;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import org.springframework.beans.factory.annotation.Autowired;

@UiController("nfp_KoatuuImportScreen")
@UiDescriptor("koatuu-import-screen.xml")
public class KoatuuImportScreen extends Screen {

    @Autowired
    private TemporaryStorage temporaryStorage;
    @Autowired
    private FileStorageUploadField fileField;
    @Autowired
    private ImportFromExcelService importFromExcelService;
    @Autowired
    private Notifications notifications;
    @Autowired
    private Messages messages;
    @Autowired
    private FileStorageUploadField dbfFileField;
    @Autowired
    private ImportFromDbfService importFromDbfService;
    @Autowired
    private ComboBox typeComboBox;

    @Subscribe
    public void onInit(InitEvent event) {
        List<String> options = Lists.newArrayList("КОАТУУ", "УКТЗЕД");
        typeComboBox.setOptionsList(options);
    }

    @Subscribe("fileField")
    public void onFileFieldFileUploadSucceed(FileUploadSucceedEvent event) throws IOException {
        File file = temporaryStorage.getFile(Objects.requireNonNull(fileField.getFileId()));
        String filePath = Objects.requireNonNull(file).getPath();
        String result = importFromExcelService.factory(filePath, "Кодифікатор", "Koatuu");
        importResultNote(result, fileField);
    }

    @Subscribe("dbfFileField")
    public void onDbfFileFieldFileUploadSucceed(FileUploadSucceedEvent event) {
        String result = null;
        File file = temporaryStorage.getFile(Objects.requireNonNull(dbfFileField.getFileId()));
        String filePath = Objects.requireNonNull(file).getAbsolutePath();
        switch (Objects.requireNonNull(typeComboBox.getValue()).toString()) {
            case "КОАТУУ":
                result = importFromDbfService.factory(filePath, "KoatuuOld");
                break;
            case "УКТЗЕД":
                result = importFromDbfService.factory(filePath, "Uktzed");
                break;
        }
        importResultNote(result, dbfFileField);
    }

    private void importResultNote(String result, FileStorageUploadField dbfFileField) {
        temporaryStorage.deleteFile(Objects.requireNonNull(dbfFileField.getFileId()));
        if (result.equals("Ok")) {
            String message = messages.getMessage("com.company.nfp.notification/successful");
            notifications.create(NotificationType.TRAY)
                    .withDescription(message)
                    .show();
            close(StandardOutcome.CLOSE);
        } else {
            String message = messages.getMessage(result);
            notifications.create(NotificationType.WARNING)
                    .withDescription(message)
                    .show();
        }
    }

    @Subscribe("closeBtn")
    public void onCloseBtnClick(ClickEvent event) {
        close(StandardOutcome.CLOSE);
    }


}