package com.company.nfp.screen.source.good.good;

import com.company.nfp.app.copy_entity.CopySourceEntityService;
import com.company.nfp.app.set_next_number.SetNextNumberService;
import com.company.nfp.entity.User;
import com.company.nfp.entity.enumerations.AccountingType;
import com.company.nfp.entity.enumerations.SourceGoodType;
import com.company.nfp.entity.source.product.GoodDetail;
import io.jmix.core.*;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.ui.Notifications;
import io.jmix.ui.action.Action;
import io.jmix.ui.component.GroupTable;
import io.jmix.ui.model.CollectionContainer;
import io.jmix.ui.screen.*;
import com.company.nfp.entity.source.product.Good;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Objects;

@UiController("nfp_Good.browse")
@UiDescriptor("good-browse.xml")
@LookupComponent("goodsTable")
public class GoodBrowse extends StandardLookup<Good> {

    @Autowired
    private GroupTable<Good> goodsTable;
    @Autowired
    private CollectionContainer<Good> goodsDc;
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private Messages messages;
    @Autowired
    private Notifications notifications;
    @Autowired
    private CopySourceEntityService copySourceEntityService;

    @Subscribe("goodsTable.copy")
    public void onGoodsTableCopy(Action.ActionPerformedEvent event) {
            var result = copySourceEntityService.copySourceEntity(getUser().getId(),
                    goodsTable.getSingleSelected().getId());
            if (result != null) {
                notificationShow("com.company.nfp.notification/successful",
                        Notifications.NotificationType.TRAY);
                goodsDc.getMutableItems().add(result);
            }
            else notificationShow("com.company.nfp.notification/invalidOperation",
                    Notifications.NotificationType.ERROR);
    }

    private User getUser() {
        User user;
        try {
            user = (User) currentAuthentication.getUser();
            return user;
        } catch (Exception e) {
            notificationShow("invalid current user", Notifications.NotificationType.ERROR);
            return null;
        }
    }

    /**
     * Show notification
     *
     * @param key              Message Bandle Key
     * @param notificationType Notification Type
     */
    private void notificationShow(String key, Notifications.NotificationType notificationType) {
        String message = messages.getMessage(key);
        notifications.create(notificationType)
                .withDescription(message)
                .show();
    }

}