package com.company.nfp.screen.source.good.good;

import com.company.nfp.app.get_entity.get_document_props.GetDocumentPropsService;
import com.company.nfp.app.numbwer_in_words.NumbersInWordsService;
import com.company.nfp.app.set_next_number.SetNextNumberService;
import com.company.nfp.entity.User;
import com.company.nfp.entity.dictionaries.counterparty.Counterparty;
import com.company.nfp.entity.enumerations.SourceGoodType;
import com.company.nfp.entity.source.product.Good;
import com.company.nfp.entity.source.product.GoodDetail;
import io.jmix.core.EntityStates;
import io.jmix.core.Messages;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.*;
import io.jmix.ui.model.CollectionContainer;
import io.jmix.ui.model.InstanceContainer;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Objects;

@UiController("nfp_Good.edit")
@UiDescriptor("good-edit.xml")
@EditedEntityContainer("goodDc")
public class GoodEdit extends StandardEditor<Good> {

    @Autowired
    private InstanceContainer<Good> goodDc;
    @Autowired
    private EntityStates entityStates;
    @Autowired
    private ComboBox<SourceGoodType> sourceGoodTypeField;
    @Autowired
    private CheckBox pnIsThereField;
    @Autowired
    private TextField<String> numberField;
    @Autowired
    private NumbersInWordsService numbersInWordsService;
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private Messages messages;
    @Autowired
    private Notifications notifications;
    @Autowired
    private TextField<BigDecimal> sumField;
    @Autowired
    private GroupBoxLayout goodDetailBox;
    @Autowired
    private SetNextNumberService setNextNumberService;
    @Autowired
    private GetDocumentPropsService getDocumentPropsService;

    @Subscribe
    public void onBeforeCommitChanges(BeforeCommitChangesEvent event) {
        if (entityStates.isNew(getEditedEntity())) {
            if (!Objects.equals(sourceGoodTypeField.getValue(), SourceGoodType.ZO_RECEIPT_INVOICE)
                    || !Objects.equals(sourceGoodTypeField.getValue(), SourceGoodType.OF_RECEIPT_INVOICE)) {
                setDocumentNumber();
            }
            if (Objects.equals(sourceGoodTypeField.getValue(), SourceGoodType.OF_EXPENSE_INVOICE)
                    || Objects.equals(sourceGoodTypeField.getValue(), SourceGoodType.OF_ACT)) {
                setTaxInvoiceNumber();
            }
            //String currentTenantId = tenantProvider.getCurrentUserTenantId();
        }
    }

    /**
     * Set the Document Number
     */
    private void setDocumentNumber() {
        Long num = setNextNumberService.setDocumentNumber(Objects.requireNonNull(sourceGoodTypeField.getValue()),
                getUser().getId());
        goodDc.getItem().setNum(num);
        goodDc.getItem().setNumber(String.valueOf(num));
        //if (currentTenantId == "no_tenant"){}
    }

    /**
     * Set the Tax Invoice Number
     */
    private void setTaxInvoiceNumber() {
        LocalDate date = LocalDate.now();
        Long taxInvoiceNum = setNextNumberService.setTaxInvoiceNumber(date, getUser().getId());
        if (taxInvoiceNum != null) {
            goodDc.getItem().setTaxInvoiceNumber(String.valueOf(taxInvoiceNum));
            goodDc.getItem().setTaxInvoiceDate(date);
        }
    }

    @Subscribe
    public void onInitEntity(InitEntityEvent<Good> event) {
        var docProps = getDocumentPropsService.getDocumentProps(getUser().getId());
        if (docProps == null) {
            notificationShow("msg://com.company.nfp.notification/invalidOperation", Notifications.NotificationType.WARNING);
        } else {
            event.getEntity().setFirmAccount(docProps.getAccount());
            event.getEntity().setEmployee(docProps.getEmployee());
            event.getEntity().setShop(docProps.getShop());
            event.getEntity().setDate(LocalDateTime.now());
        }
    }

    /**
     * Screen Elements setting according to the Document Type.
     */
    @Subscribe("sourceGoodTypeField")
    public void onSourceGoodTypeFieldValueChange(HasValue.ValueChangeEvent<SourceGoodType> event) {
        //try {
        switch (Objects.requireNonNull(event.getValue())) {
            case OF_EXPENSE_INVOICE:
            case OF_ACT:
                if (entityStates.isNew(getEditedEntity())) {
                    //pnIsThereField.setEditable(true);
                    Boolean isVatPayer = getEditedEntity().getFirmAccount().getFirm().getIsVatPayer();
                    try {
                        if (isVatPayer) pnIsThereField.setValue(true);
                        else pnIsThereField.setValue(false);
                    } catch (Exception e) {
                        pnIsThereField.setValue(false);
                    }
                }
                sumField.setValue(null);
                sumField.setEditable(false);
                goodDetailBox.setVisible(true);
                break;
            case ZO_RECEIPT_INVOICE:
            case OF_RECEIPT_INVOICE:
                numberField.setVisible(true);
                break;
            case ZO_CASH:
            case OF_CASH:
            case ZO_BANK:
            case OF_BANK:
                sumField.setEditable(true);
                goodDetailBox.setVisible(false);
                break;
            default:
                if (entityStates.isNew(getEditedEntity())) {
                    pnIsThereField.setValue(null);
                    pnIsThereField.setEditable(false);
                }
                sumField.setValue(null);
                sumField.setEditable(false);
                goodDetailBox.setVisible(true);
                numberField.setVisible(false);
                numberField.setValue(null);
                break;
        }
    }

    @Subscribe("counterpartyField")
    public void onCounterpartyFieldValueChange(HasValue.ValueChangeEvent<Counterparty> event) {
        if (getEditedEntity().getSourceGoodType() == SourceGoodType.OF_RECEIPT_INVOICE) {
            if (entityStates.isNew(getEditedEntity())) {
                try {
                    Boolean isVatPayer = Objects.requireNonNull(event.getValue()).getIsVatPayer();
                    if (isVatPayer) pnIsThereField.setValue(true);
                    else pnIsThereField.setValue(false);
                } catch (Exception e) {
                    pnIsThereField.setValue(false);
                }
            }
        }
    }

    /**
     * Document Amounts calculation and setting
     */
    @Subscribe(id = "goodDetailDc", target = Target.DATA_CONTAINER)
    public void onGoodDetailDcCollectionChange(CollectionContainer.CollectionChangeEvent<GoodDetail> event) {
        var num = 1;
        var sumPrice = BigDecimal.ZERO;
        var sumTax = BigDecimal.ZERO;
        var sumWithoutTax = BigDecimal.ZERO;
        var sum = BigDecimal.ZERO;
        var balWithoutTax = BigDecimal.ZERO;

        try {
            if (event.getChanges().size() != 0) {
                for (GoodDetail detail : getEditedEntity().getGoodDetail()) {
                    detail.setNumberInOrder(num++);
                    sumPrice = sumPrice.add(detail.getSumPrice());
                    sumTax = sumTax.add(detail.getSumTax());
                    sumWithoutTax = sumWithoutTax.add(detail.getSumWithoutTax());
                    sum = sum.add(detail.getSum());
                }
                var typeGood = getEditedEntity().getSourceGoodType();
                switch (typeGood) {
                    case ZO_EXPENSE_INVOICE:
                    case OF_EXPENSE_INVOICE:
                    case ZO_ACT:
                    case OF_ACT:
                    case ZO_CHECK:
                    case OF_CHECK:
                        getEditedEntity().setSumPrice(sumPrice);
                        getEditedEntity().setSumTax(sumTax);
                        getEditedEntity().setSumWithoutTax(sumWithoutTax);
                        getEditedEntity().setSum(sum);
                        getEditedEntity().setDifference(sum.subtract(sumPrice));
                        getEditedEntity().setSumInWords(numbersInWordsService.moneyToStr(sum));

                        balWithoutTax = BigDecimal.ZERO.subtract(sumWithoutTax);
                        getEditedEntity().setBalWithoutTax(balWithoutTax);
                        getEditedEntity().setBalTax(BigDecimal.ZERO.subtract(sumTax));
                        getEditedEntity().setBal(BigDecimal.ZERO.subtract(sum));
                        break;
                    case OF_RECEIPT_INVOICE:
                    case ZO_RECEIPT_INVOICE:
                        getEditedEntity().setSumPrice(sumPrice);
                        getEditedEntity().setSumTax(sumTax);
                        getEditedEntity().setSumWithoutTax(sumWithoutTax);
                        getEditedEntity().setSum(sum);
                        getEditedEntity().setDifference(sum.subtract(sumPrice));
                        getEditedEntity().setSumInWords(numbersInWordsService.moneyToStr(sum));

                        getEditedEntity().setBalWithoutTax(sumWithoutTax);
                        getEditedEntity().setBalTax(sumTax);
                        getEditedEntity().setBal(sum);
                        break;
                    case ZO_BANK:
                    case OF_BANK:
                    case ZO_CASH:
                    case OF_CASH:
                        sum = getEditedEntity().getSum();
                        break;
                }
            }
        } catch (Exception e) {
            notificationShow("invalid amount calculation", Notifications.NotificationType.ERROR);
        }
        try {
            if (getEditedEntity().getStat() != null) {
                var typeMoney = getEditedEntity().getStat().getDirection();
                switch (typeMoney) {
                    case PROFIT:
                        getEditedEntity().setBal(sum);
                        break;
                    case EXPENSE:
                        getEditedEntity().setBal(BigDecimal.ZERO.subtract(sum));
                        break;
                }
            }
        } catch (Exception e) {
            notificationShow("invalid set balance by Stat", Notifications.NotificationType.ERROR);
        }
    }

    private User getUser() {
        User user;
        try {
            user = (User) currentAuthentication.getUser();
            return user;
        } catch (Exception e) {
            notificationShow("invalid current user", Notifications.NotificationType.ERROR);
            return null;
        }
    }

    /**
     * Show notification
     *
     * @param key              Message Bandle Key
     * @param notificationType Notification Type
     */
    private void notificationShow(String key, Notifications.NotificationType notificationType) {
        String message = messages.getMessage(key);
        notifications.create(notificationType)
                .withDescription(message)
                .show();
    }
}