package com.company.nfp.screen.source.good.good_detail;

import com.company.nfp.app.calculation.product_calculations.ProductQuantityBalanceService;
import com.company.nfp.app.get_entity.get_last_purchase.GetLastCostsOfService;
import com.company.nfp.app.get_entity.get_last_purchase.GetLastCostsZoService;
import com.company.nfp.entity.User;
import com.company.nfp.entity.dictionaries.product.Product;
import com.company.nfp.entity.enumerations.AccountingType;
import com.company.nfp.entity.enumerations.SourceGoodType;
import io.jmix.core.Messages;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.ui.Notifications;
import io.jmix.ui.component.Button;
import io.jmix.ui.component.HasValue;
import io.jmix.ui.component.TextField;
import io.jmix.ui.screen.*;
import com.company.nfp.entity.source.product.GoodDetail;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.Map;
import java.util.UUID;

@UiController("nfp_GoodDetail.edit")
@UiDescriptor("good-detail-edit.xml")
@EditedEntityContainer("goodDetailDc")
public class GoodDetailEdit extends StandardEditor<GoodDetail> {
    @Autowired
    private GetLastCostsZoService getLastCostsZoService;
    @Autowired
    private TextField<BigDecimal> costPriceField;
    @Autowired
    private TextField<BigDecimal> costField;
    @Autowired
    private Notifications notifications;
    @Autowired
    private Messages messages;
    @Autowired
    private ProductQuantityBalanceService productQuantityBalanceService;
    @Autowired
    private GetLastCostsOfService getLastCostsOfService;
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private Button withExtraChargeBtn;

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        if (getEditedEntity().getGood().getSourceGoodType() == SourceGoodType.OF_RECEIPT_INVOICE ||
                getEditedEntity().getGood().getSourceGoodType() == SourceGoodType.ZO_RECEIPT_INVOICE) {
            withExtraChargeBtn.setVisible(true);
        }
    }

    @Subscribe("byLastPurchaseBtn")
    public void onByLastPurchaseBtnClick(Button.ClickEvent event) {
        Product product = getEditedEntity().getProduct();
        LocalDateTime docDate = getEditedEntity().getGood().getDate();
        if (product == null || docDate == null) {
            notificationShow("com.company.nfp.notification/notEmpty", Notifications.NotificationType.WARNING);
            return;
        }
        Map<String, BigDecimal> map;
        try {
            if (getEditedEntity().getGood().getAccountingType() == AccountingType.ZO) {
                map = getLastCostsZoService.getLastCostsZo(product.getId(), docDate);
            } else {
                map = getLastCostsOfService.getLastCostsOf(getUser().getId(), product.getId(), docDate);
            }
            costPriceField.setValue(map.get("costPrice"));
            costField.setValue(map.get("cost"));
            notificationShow("com.company.nfp.notification/successful", Notifications.NotificationType.TRAY);
        } catch (Exception e) {
            notificationShow("com.company.nfp.notification/notGoodPurchase", Notifications.NotificationType.HUMANIZED);
        }
    }

    @Subscribe("withExtraChargeBtn")
    public void onWithExtraChargeBtnClick(Button.ClickEvent event) {
        var costPrice = getEditedEntity().getCostPrice();
        var extraCharge = getEditedEntity().getGood().getExtraCharge();
        if (costPrice == null || extraCharge == null) {
            notificationShow("com.company.nfp.notification/notEmpty", Notifications.NotificationType.WARNING);
            return;
        }
        var cost = costPrice.multiply(extraCharge);
        var costRound = cost.setScale(1, RoundingMode.HALF_UP);
        if (getEditedEntity().getGood().getSourceGoodType() == SourceGoodType.ZO_RECEIPT_INVOICE)
            getEditedEntity().setCost(costRound);
        else getEditedEntity().setCost(cost);
    }

    @Subscribe
    public void onBeforeCommitChanges(BeforeCommitChangesEvent event) {
        BigDecimal tax = null;
        BigDecimal costWithoutTax = null;
        BigDecimal sumPrice = null;
        BigDecimal sum = null;
        BigDecimal sumTax = null;
        BigDecimal sumWithoutTax = null;


        var quantity = getEditedEntity().getQuantity();
        var costPrice = getEditedEntity().getCostPrice();
        var cost = getEditedEntity().getCost();
        var type = getEditedEntity().getGood().getSourceGoodType();
        switch (type) {
            case ZO_EXPENSE_INVOICE:
            case OF_EXPENSE_INVOICE:
            case ZO_ACT:
            case OF_ACT:
            case ZO_CHECK:
            case OF_CHECK:
                var balQuantity = BigDecimal.ZERO.subtract(quantity);
                getEditedEntity().setBalQuantity(balQuantity);
                
                tax = cost.divide(new BigDecimal("6"), RoundingMode.HALF_UP);
                costWithoutTax = cost.subtract(tax);
                sumPrice = quantity.multiply(costPrice);
                sum = quantity.multiply(cost);
                sumTax = quantity.multiply(tax);
                sumWithoutTax = sum.subtract(sumTax);

                break;
            case OF_RECEIPT_INVOICE:
            case ZO_RECEIPT_INVOICE:
                getEditedEntity().setBalQuantity(quantity);

                tax = costPrice.divide(new BigDecimal("6"), RoundingMode.HALF_UP);
                costWithoutTax = costPrice.subtract(tax);
                sumPrice = quantity.multiply(costWithoutTax);
                sum = quantity.multiply(costPrice);
                sumTax = quantity.multiply(tax);
                sumWithoutTax = sum.subtract(sumTax);


                break;
        }

        getEditedEntity().setSumPrice(sumPrice);
        getEditedEntity().setCostWithoutTax(costWithoutTax);
        getEditedEntity().setTax(tax);
        getEditedEntity().setSumWithoutTax(sumWithoutTax);
        getEditedEntity().setSumTax(sumTax);
        getEditedEntity().setSum(sum);

    }

    /**
     * Show notification
     *
     * @param key              Message Bandle Key
     * @param notificationType Notification Type
     */
    private void notificationShow(String key, Notifications.NotificationType notificationType) {
        String message = messages.getMessage(key);
        notifications.create(notificationType)
                .withDescription(message)
                .show();
    }

    @Subscribe("productField")
    public void onProductFieldValueChange(HasValue.ValueChangeEvent<Product> event) {
        var accountingType = getEditedEntity().getGood().getAccountingType();
        var productId = getEditedEntity().getProduct().getId();
        var currentDate = getEditedEntity().getGood().getDate();
        var balance = productQuantityBalanceCalc(accountingType, productId, currentDate);
        getEditedEntity().setBalance(balance);
    }

    private BigDecimal productQuantityBalanceCalc(AccountingType accountingType, UUID productId, LocalDateTime dateTime) {
        BigDecimal result = BigDecimal.ZERO;
        switch (accountingType) {
            case ZO:
                result = productQuantityBalanceService.productQuantityBalance(productId, dateTime);
                break;
            case OF:
                var firmId = getEditedEntity().getGood().getFirmAccount().getFirm().getId();
                result = productQuantityBalanceService.productQuantityBalance(firmId, productId, dateTime);
                break;
        }
        return result;
    }

    public User getUser() {
        return (User) currentAuthentication.getUser();
    }


}