package com.company.nfp.screen.source.good.revaluation;

import com.company.nfp.app.calculation.product_calculations.CalcProductCostPerDateService;
import com.company.nfp.entity.User;
import com.company.nfp.entity.dictionaries.firm.Firm;
import com.company.nfp.entity.dictionaries.product.Product;
import com.company.nfp.entity.enumerations.SourceRevaluationType;
import com.company.nfp.entity.not_persistent.revaluation.CostsPerDateEntity;
import io.jmix.core.Messages;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.ui.Notifications;
import io.jmix.ui.action.Action;
import io.jmix.ui.component.*;
import io.jmix.ui.screen.*;
import com.company.nfp.entity.source.revaluation.Revaluation;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.time.LocalDateTime;

@UiController("nfp_Revaluation.edit")
@UiDescriptor("revaluation-edit.xml")
@EditedEntityContainer("revaluationDc")
public class RevaluationEdit extends StandardEditor<Revaluation> {
    @Autowired
    private EntityPicker<Product> productField;
    @Autowired
    private Messages messages;
    @Autowired
    private Notifications notifications;
    @Autowired
    private DateField<LocalDateTime> dateField;
    @Autowired
    private TextField<BigDecimal> costPriceField;
    @Autowired
    private TextField<BigDecimal> oldCostField;
    @Autowired
    private CalcProductCostPerDateService calcProductCostPerDateService;
    @Autowired
    private ComboBox<SourceRevaluationType> sourceRevaluationTypeField;
    @Autowired
    private CurrentAuthentication currentAuthentication;
    @Autowired
    private EntityPicker<Firm> firmField;

    @Subscribe("byLastPurchase")
    public void onByLastPurchase(Action.ActionPerformedEvent event) {
        if (productField.getValue() == null) {
            notificationShow("msg:///notifications.warning.entryProductNeed",
                    Notifications.NotificationType.WARNING);
            productField.focus();
        } else if (dateField.getValue() == null) {
            notificationShow("msg:///notifications.warning.periodDatesNotEntry",
                    Notifications.NotificationType.WARNING);
            dateField.focus();
        } else {
            CostsPerDateEntity costsPerDate;
            switch (getEditedEntity().getSourceRevaluationType()) {
                case ZO_REVALUATION:
                    costsPerDate = calcProductCostPerDateService.calcProductCostPerDate(null,
                            getEditedEntity().getProduct().getId(),
                            getEditedEntity().getDate()
                    );
                    setValueFields(costsPerDate);
                    break;
                case OF_REVALUATION:
                    costsPerDate = calcProductCostPerDateService.calcProductCostPerDate(getUser().getId(),
                            getEditedEntity().getProduct().getId(),
                            getEditedEntity().getDate());
                    setValueFields(costsPerDate);
                    break;
                default:
                    notificationShow("msg:///notifications.warning.dataNotEntry", Notifications.NotificationType.WARNING);
                    break;
            }
        }
    }

    @Subscribe("sourceRevaluationTypeField")
    public void onSourceRevaluationTypeFieldValueChange(HasValue.ValueChangeEvent<SourceRevaluationType> event) {
        if (sourceRevaluationTypeField.getValue() == SourceRevaluationType.OF_REVALUATION) {
            firmField.setVisible(true);
        } else {
            firmField.setVisible(false);
            getEditedEntity().setFirm(null);
        }
    }

    private void setValueFields(CostsPerDateEntity costsPerDate) {
        if (costsPerDate.getCostPrice() == null && costsPerDate.getCost() == null) {
            notificationShow("msg:///notifications.warning.productNoPurchased", Notifications.NotificationType.WARNING);
        }
        costPriceField.setValue(costsPerDate.getCostPrice());
        oldCostField.setValue(costsPerDate.getCost());
    }

    private User getUser() {
        User user;
        try {
            user = (User) currentAuthentication.getUser();
            return user;
        } catch (Exception e) {
            notificationShow("invalid current user", Notifications.NotificationType.ERROR);
            return null;
        }
    }


    /**
     * Show notification
     *
     * @param key              Message Bandle Key
     * @param notificationType Notification Type
     */
    private void notificationShow(String key, Notifications.NotificationType notificationType) {
        String message = messages.getMessage(key);
        notifications.create(notificationType)
                .withDescription(message)
                .show();
    }

}