package com.company.nfp.screen.user;

import com.company.nfp.entity.User;
import com.google.common.base.Strings;
import io.jmix.core.EntityStates;
import io.jmix.core.security.event.SingleUserPasswordChangeEvent;
import io.jmix.multitenancy.core.TenantProvider;
import io.jmix.multitenancyui.MultitenancyUiSupport;
import io.jmix.ui.Notifications;
import io.jmix.ui.Notifications.NotificationType;
import io.jmix.ui.component.ComboBox;
import io.jmix.ui.component.HasValue.ValueChangeEvent;
import io.jmix.ui.component.PasswordField;
import io.jmix.ui.component.TextField;
import io.jmix.ui.model.DataContext;
import io.jmix.ui.model.DataContext.PostCommitEvent;
import io.jmix.ui.model.InstanceContainer;
import io.jmix.ui.navigation.Route;
import io.jmix.ui.screen.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;

import java.util.Arrays;
import java.util.Objects;
import java.util.TimeZone;


@UiController("nfp_User.edit")
@UiDescriptor("user-edit.xml")
@EditedEntityContainer("userDc")
@Route(value = "users/edit", parentPrefix = "users")
public class UserEdit extends StandardEditor<User> {

  @Autowired
  private EntityStates entityStates;

  @Autowired
  private PasswordEncoder passwordEncoder;

  @Autowired
  private PasswordField passwordField;

  @Autowired
  private TextField<String> usernameField;

  @Autowired
  private PasswordField confirmPasswordField;

  @Autowired
  private Notifications notifications;

  @Autowired
  private MessageBundle messageBundle;

  @Autowired
  private ComboBox<String> timeZoneField;

  private boolean isNewEntity;
  @Autowired
  private TenantProvider tenantProvider;
  @Autowired
  private ComboBox<String> tenantField;
  @Autowired
  private MultitenancyUiSupport multitenancyUiSupport;
  @Autowired
  private InstanceContainer<User> userDc;

  @Subscribe
  public void onInitEntity(InitEntityEvent<User> event) {
    tenantField.setEditable(true);
    usernameField.setEditable(true);
    passwordField.setVisible(true);
    confirmPasswordField.setVisible(true);
    isNewEntity = true;
  }

  @Subscribe
  public void onBeforeShow(BeforeShowEvent event) {
    String currentTenantId = tenantProvider.getCurrentUserTenantId();
    if (!currentTenantId.equals(TenantProvider.NO_TENANT)
        && Strings.isNullOrEmpty(tenantField.getValue())) {
      tenantField.setEditable(false);
      tenantField.setValue(currentTenantId);
    }
  }


  @Subscribe
  public void onAfterShow(AfterShowEvent event) {
    if (entityStates.isNew(getEditedEntity())) {
      usernameField.focus();
    }
  }

  @Subscribe
  protected void onBeforeCommit(BeforeCommitChangesEvent event) {
    if (entityStates.isNew(getEditedEntity())) {
      if (!Objects.equals(passwordField.getValue(), confirmPasswordField.getValue())) {
        notifications.create(NotificationType.WARNING)
            .withCaption(messageBundle.getMessage("passwordsDoNotMatch"))
            .show();
        event.preventCommit();
      }
      getEditedEntity().setPassword(passwordEncoder.encode(passwordField.getValue()));
    }
  }

  @Subscribe(target = Target.DATA_CONTEXT)
  public void onPostCommit(PostCommitEvent event) {
    if (isNewEntity) {
      getApplicationContext().publishEvent(
          new SingleUserPasswordChangeEvent(getEditedEntity().getUsername(),
              passwordField.getValue()));
    }
  }

  @Subscribe
  public void onInit(InitEvent event) {
    tenantField.setOptionsList(multitenancyUiSupport.getTenantOptions());
    timeZoneField.setOptionsList(Arrays.asList(TimeZone.getAvailableIDs()));
  }

  @Subscribe("tenantField")
  public void onTenantFieldValueChange(ValueChangeEvent<String> event) {
/*
    var fullName = userDc.getItem().getUsername();
    assert fullName != null;
    var login = fullName.substring(fullName.indexOf("|"));
    var userName = event.getValue() + login;
*/
    //var userName = multitenancyUiSupport.getUsernameByTenant(usernameField.getValue(), event.getValue());
    var userName = multitenancyUiSupport.getUsernameByTenant(userDc.getItem().getUsername(), event.getValue());
/*
    var name = Objects.requireNonNull(usernameField.getValue())
            .substring(usernameField.getValue().indexOf("|") + 1);
    var userName = String.format("%s%s%s", event.getValue(), "|", name);
*/
    usernameField.setValue(userName);
  }
}