package com.company.nfp.security;

import com.company.nfp.entity.User;
import com.company.nfp.entity.dictionaries.counterparty.Counterparty;
import com.company.nfp.entity.dictionaries.counterparty.counterparty_account.CounterpartyAccount;
import com.company.nfp.entity.dictionaries.employee.Employee;
import com.company.nfp.entity.dictionaries.firm.Firm;
import com.company.nfp.entity.dictionaries.firm.Shop;
import com.company.nfp.entity.dictionaries.firm.ShopOption;
import com.company.nfp.entity.dictionaries.firm.firm_account.FirmAccount;
import com.company.nfp.entity.dictionaries.product.Product;
import com.company.nfp.entity.dictionaries.product.ProductSubgroup;
import com.company.nfp.entity.dictionaries.product.Uktzed;
import com.company.nfp.entity.dictionaries.product.Unit;
import com.company.nfp.entity.dictionaries.prro.Prro;
import com.company.nfp.entity.not_persistent.prro.PrroStateEntity;
import com.company.nfp.entity.not_persistent.prro.SaleReturnReceiptEntity;
import com.company.nfp.entity.source.product.Good;
import com.company.nfp.entity.source.product.GoodDetail;
import com.company.nfp.entity.source.product.GoodDetailExcise;
import com.company.nfp.entity.source.revaluation.Revaluation;
import io.jmix.security.model.EntityAttributePolicyAction;
import io.jmix.security.model.EntityPolicyAction;
import io.jmix.security.role.annotation.EntityAttributePolicy;
import io.jmix.security.role.annotation.EntityPolicy;
import io.jmix.security.role.annotation.ResourceRole;
import io.jmix.security.role.annotation.SpecificPolicy;
import io.jmix.securityui.role.annotation.MenuPolicy;
import io.jmix.securityui.role.annotation.ScreenPolicy;

@ResourceRole(name = "MultitenantCashierRole", code = "multitenant-cashier-role")
public interface MultitenantCashierRole {
    @MenuPolicy(menuIds = "nfp_CashierCheckStartScreen")
    @ScreenPolicy(screenIds = {"nfp_CashierCheckStartScreen", "nfp_CheckScreen", "nfp_CheckViewScreen", "nfp_ExciseScreen", "nfp_ServiceInOutScreen", "nfp_XreportScreen"})
    void screens();

    @EntityPolicy(entityClass = Counterparty.class, actions = EntityPolicyAction.READ)
    @EntityAttributePolicy(entityClass = Counterparty.class, attributes = {"edrpou", "shortName", "fullName", "id"}, action = EntityAttributePolicyAction.VIEW)
    void counterparty();

    @EntityAttributePolicy(entityClass = CounterpartyAccount.class, attributes = {"account", "main", "id"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = CounterpartyAccount.class, actions = EntityPolicyAction.READ)
    void counterpartyAccount();

    @EntityAttributePolicy(entityClass = Employee.class, attributes = {"fullName", "shortName", "shop", "user", "main", "prro", "id"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = Employee.class, actions = EntityPolicyAction.READ)
    void employee();

    @EntityAttributePolicy(entityClass = Firm.class, attributes = {"fullName", "shortName", "id"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = Firm.class, actions = EntityPolicyAction.READ)
    void firm();

    @EntityAttributePolicy(entityClass = FirmAccount.class, attributes = {"id", "account", "main", "bank", "userId", "firm"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = FirmAccount.class, actions = EntityPolicyAction.READ)
    void firmAccount();

    @EntityAttributePolicy(entityClass = Good.class, attributes = {"id", "accountingType", "sourceGoodType", "pnIsThere", "counterparty", "firmAccount", "shop", "employee", "stat", "goodDetail", "num", "number", "date", "reason", "docSubType", "docSubTypeString", "fn", "qr", "dfsLink", "email", "change", "byCash", "byCredit", "byCheck", "byCard", "byBonuses", "checkText", "orderDoc", "payer", "taxInvoiceNumber", "taxInvoiceDate", "contractNumber", "contractDate", "saleCondition", "accepted", "proxy", "extraCharge", "sumPrice", "sumWithoutTax", "sumTax", "sum", "balWithoutTax", "balTax", "bal", "difference", "sumInWords", "comment"}, action = EntityAttributePolicyAction.MODIFY)
    @EntityAttributePolicy(entityClass = Good.class, attributes = {"tenantAttribute", "version", "createdBy", "createdDate", "lastModifiedBy", "lastModifiedDate", "deletedBy", "deletedDate"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = Good.class, actions = {EntityPolicyAction.CREATE, EntityPolicyAction.READ, EntityPolicyAction.UPDATE})
    void good();

    @EntityAttributePolicy(entityClass = GoodDetail.class, attributes = {"id", "numberInOrder", "product", "balance", "quantity", "balQuantity", "costPrice", "costWithoutTax", "tax", "cost", "sumPrice", "sumWithoutTax", "sumTax", "sum", "good", "excise"}, action = EntityAttributePolicyAction.MODIFY)
    @EntityPolicy(entityClass = GoodDetail.class, actions = {EntityPolicyAction.CREATE, EntityPolicyAction.READ, EntityPolicyAction.UPDATE})
    void goodDetail();

    @EntityAttributePolicy(entityClass = GoodDetailExcise.class, attributes = {"id", "excise", "goodDetail"}, action = EntityAttributePolicyAction.MODIFY)
    @EntityPolicy(entityClass = GoodDetailExcise.class, actions = {EntityPolicyAction.CREATE, EntityPolicyAction.READ, EntityPolicyAction.UPDATE})
    void goodDetailExcise();

    @EntityAttributePolicy(entityClass = Product.class, attributes = {"id", "fullName", "shortName", "barcode", "code", "vatGroup", "isExcise", "isService", "productSubgroup", "uktzed", "unit", "balance", "costPrice", "costPurchase"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = Product.class, actions = EntityPolicyAction.READ)
    void product();

    @EntityAttributePolicy(entityClass = ProductSubgroup.class, attributes = {"id", "fullName", "shortName"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = ProductSubgroup.class, actions = EntityPolicyAction.READ)
    void productSubgroup();

    @EntityAttributePolicy(entityClass = Prro.class, attributes = {"id", "fullName", "shortName", "host", "port", "pathReceipt", "employee", "activeUntil"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = Prro.class, actions = EntityPolicyAction.READ)
    void prro();

    @EntityAttributePolicy(entityClass = PrroStateEntity.class, attributes = "*", action = EntityAttributePolicyAction.MODIFY)
    @EntityPolicy(entityClass = PrroStateEntity.class, actions = EntityPolicyAction.ALL)
    void prroStateEntity();

    @EntityAttributePolicy(entityClass = Revaluation.class, attributes = {"id", "sourceRevaluationType", "firm", "product", "date", "costPrice", "oldCost", "cost"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = Revaluation.class, actions = EntityPolicyAction.READ)
    void revaluation();

    @EntityAttributePolicy(entityClass = SaleReturnReceiptEntity.class, attributes = "*", action = EntityAttributePolicyAction.MODIFY)
    @EntityPolicy(entityClass = SaleReturnReceiptEntity.class, actions = EntityPolicyAction.ALL)
    void saleReturnReceiptEntity();

    @EntityAttributePolicy(entityClass = Shop.class, attributes = {"id", "fullName", "shortName", "firm", "employees", "address", "main"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = Shop.class, actions = EntityPolicyAction.READ)
    void shop();

    @EntityAttributePolicy(entityClass = ShopOption.class, attributes = {"id", "shop", "salesMethod"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = ShopOption.class, actions = EntityPolicyAction.READ)
    void shopOption();

    @EntityAttributePolicy(entityClass = Uktzed.class, attributes = {"id", "fullName", "shortName", "code", "num"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = Uktzed.class, actions = EntityPolicyAction.READ)
    void uktzed();

    @EntityAttributePolicy(entityClass = Unit.class, attributes = {"id", "fullName", "shortName", "code", "num", "latName", "quantity"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = Unit.class, actions = EntityPolicyAction.READ)
    void unit();

    @EntityAttributePolicy(entityClass = User.class, attributes = {"id", "username", "firstName", "lastName", "employee"}, action = EntityAttributePolicyAction.VIEW)
    @EntityPolicy(entityClass = User.class, actions = EntityPolicyAction.READ)
    void user();

    @SpecificPolicy(resources = "ui.loginToUi")
    void specific();
}